''' unit test template for ONTAP Ansible module '''

from __future__ import print_function
import json
import pytest

import time
from units.compat import unittest
from units.compat.mock import patch, Mock
from ansible.module_utils import basic
from ansible.module_utils._text import to_bytes
import ansible.module_utils.netapp as netapp_utils

from ansible.modules.storage.netapp.na_ontap_service_processor_network \
    import NetAppOntapServiceProcessorNetwork as sp_module  # module under test

if not netapp_utils.has_netapp_lib():
    pytestmark = pytest.mark.skip('skipping as missing required netapp_lib')


def set_module_args(args):
    """prepare arguments so that they will be picked up during module creation"""
    args = json.dumps({'ANSIBLE_MODULE_ARGS': args})
    basic._ANSIBLE_ARGS = to_bytes(args)  # pylint: disable=protected-access


class AnsibleExitJson(Exception):
    """Exception class to be raised by module.exit_json and caught by the test case"""
    pass


class AnsibleFailJson(Exception):
    """Exception class to be raised by module.fail_json and caught by the test case"""
    pass


def exit_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over exit_json; package return data into an exception"""
    if 'changed' not in kwargs:
        kwargs['changed'] = False
    raise AnsibleExitJson(kwargs)


def fail_json(*args, **kwargs):  # pylint: disable=unused-argument
    """function to patch over fail_json; package return data into an exception"""
    kwargs['failed'] = True
    raise AnsibleFailJson(kwargs)


class MockONTAPConnection(object):
    ''' mock server connection to ONTAP host '''

    def __init__(self, kind=None, data=None):
        ''' save arguments '''
        self.kind = kind
        self.data = data
        self.xml_in = None
        self.xml_out = None

    def invoke_successfully(self, xml, enable_tunneling):  # pylint: disable=unused-argument
        ''' mock invoke_successfully returning xml data '''
        self.xml_in = xml
        if self.kind == 'sp-enabled':
            xml = self.build_sp_info(self.data)
        elif self.kind == 'sp-disabled':
            xml = self.build_sp_disabled_info(self.data)
        else:
            xml = self.build_info()
        self.xml_out = xml
        return xml

    @staticmethod
    def build_sp_info(sp):
        ''' build xml data for vserser-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 1,
            'attributes-list':
            {
                'service-processor-network-info':
                {
                    'node-name': sp['node'],
                    'is-enabled': 'true',
                    'address-type': sp['address_type'],
                    'dhcp': 'v4',
                    'gateway-ip-address': sp['gateway_ip_address'],
                    'netmask': sp['netmask'],
                    'ip-address': sp['ip_address'],
                    'setup-status': 'succeeded',
                }
            }
        }
        xml.translate_struct(attributes)
        return xml

    @staticmethod
    def build_info():
        ''' build xml data for vserser-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 0
        }
        xml.translate_struct(attributes)
        return xml

    @staticmethod
    def build_sp_disabled_info(sp):
        ''' build xml data for vserser-info '''
        xml = netapp_utils.zapi.NaElement('xml')
        attributes = {
            'num-records': 1,
            'attributes-list':
            {
                'service-processor-network-info':
                {
                    'node-name': sp['node'],
                    'is-enabled': 'false',
                    'address-type': sp['address_type'],
                    'setup-status': 'not_setup',
                }
            }
        }
        xml.translate_struct(attributes)
        return xml


class TestMyModule(unittest.TestCase):
    ''' a group of related Unit Tests '''

    def setUp(self):
        self.mock_module_helper = patch.multiple(basic.AnsibleModule,
                                                 exit_json=exit_json,
                                                 fail_json=fail_json)
        self.mock_module_helper.start()
        self.addCleanup(self.mock_module_helper.stop)
        self.server = MockONTAPConnection()
        self.mock_sp = {
            'node': 'test-vsim1',
            'gateway_ip_address': '2.2.2.2',
            'address_type': 'ipv4',
            'ip_address': '1.1.1.1',
            'netmask': '255.255.248.0',
            'dhcp': 'v4'
        }

    def mock_args(self, enable=True):
        data = {
            'node': self.mock_sp['node'],
            'is_enabled': enable,
            'address_type': self.mock_sp['address_type'],
            'hostname': 'host',
            'username': 'admin',
            'password': 'password',
        }
        if enable is True:
            data['ip_address'] = self.mock_sp['ip_address']
            data['gateway_ip_address'] = self.mock_sp['gateway_ip_address']
            data['netmask'] = self.mock_sp['netmask']
            data['dhcp'] = 'v4'
        return data

    def get_sp_mock_object(self, kind=None):
        """
        Helper method to return an na_ontap_volume object
        :param kind: passes this param to MockONTAPConnection()
        :return: na_ontap_volume object
        """
        sp_obj = sp_module()
        sp_obj.autosupport_log = Mock(return_value=None)
        if kind is None:
            sp_obj.server = MockONTAPConnection()
        else:
            sp_obj.server = MockONTAPConnection(kind=kind, data=self.mock_sp)
        return sp_obj

    def test_module_fail_when_required_args_missing(self):
        ''' required arguments are reported as errors '''
        with pytest.raises(AnsibleFailJson) as exc:
            set_module_args({})
            sp_module()
        print('Info: %s' % exc.value.args[0]['msg'])

    def test_modify_error_on_disabled_sp(self):
        ''' a more interesting test '''
        data = self.mock_args(enable=False)
        data['ip_address'] = self.mock_sp['ip_address']
        set_module_args(data)
        with pytest.raises(AnsibleFailJson) as exc:
            self.get_sp_mock_object('sp-disabled').apply()
        assert 'Error: Cannot modify a service processor network if it is disabled.' in \
               exc.value.args[0]['msg']

    def test_modify_sp(self):
        ''' a more interesting test '''
        data = self.mock_args()
        data['ip_address'] = '3.3.3.3'
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_sp_mock_object('sp-enabled').apply()
        assert exc.value.args[0]['changed']

    def test_modify_sp_wait(self):
        ''' a more interesting test '''
        data = self.mock_args()
        data['ip_address'] = '3.3.3.3'
        data['wait_for_completion'] = True
        set_module_args(data)
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_sp_mock_object('sp-enabled').apply()
        assert exc.value.args[0]['changed']

    @patch('ansible.modules.storage.netapp.na_ontap_service_processor_network.NetAppOntapServiceProcessorNetwork.'
           'get_service_processor_network')
    def test_non_existing_sp(self, get_sp):
        set_module_args(self.mock_args())
        get_sp.return_value = None
        with pytest.raises(AnsibleFailJson) as exc:
            self.get_sp_mock_object().apply()
        assert 'Error No Service Processor for node: test-vsim1' in exc.value.args[0]['msg']

    @patch('ansible.modules.storage.netapp.na_ontap_service_processor_network.NetAppOntapServiceProcessorNetwork.'
           'get_sp_network_status')
    @patch('time.sleep')
    def test_wait_on_sp_status(self, get_sp, sleep):
        data = self.mock_args()
        data['gateway_ip_address'] = '4.4.4.4'
        data['wait_for_completion'] = True
        set_module_args(data)
        get_sp.side_effect = ['in_progress', 'done']
        with pytest.raises(AnsibleExitJson) as exc:
            self.get_sp_mock_object('sp-enabled').apply()
        sleep.assert_called_once_with()
        assert exc.value.args[0]['changed']



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_webfilter_profile
short_description: Configure Web filter profiles.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure webfilter feature and profile category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    webfilter_profile:
        description:
            - Configure Web filter profiles.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            comment:
                description:
                    - Optional comments.
            extended-log:
                description:
                    - Enable/disable extended logging for web filtering.
                choices:
                    - enable
                    - disable
            ftgd-wf:
                description:
                    - FortiGuard Web Filter settings.
                suboptions:
                    exempt-quota:
                        description:
                            - Do not stop quota for these categories.
                    filters:
                        description:
                            - FortiGuard filters.
                        suboptions:
                            action:
                                description:
                                    - Action to take for matches.
                                choices:
                                    - block
                                    - authenticate
                                    - monitor
                                    - warning
                            auth-usr-grp:
                                description:
                                    - Groups with permission to authenticate.
                                suboptions:
                                    name:
                                        description:
                                            - User group name. Source user.group.name.
                                        required: true
                            category:
                                description:
                                    - Categories and groups the filter examines.
                            id:
                                description:
                                    - ID number.
                                required: true
                            log:
                                description:
                                    - Enable/disable logging.
                                choices:
                                    - enable
                                    - disable
                            override-replacemsg:
                                description:
                                    - Override replacement message.
                            warn-duration:
                                description:
                                    - Duration of warnings.
                            warning-duration-type:
                                description:
                                    - Re-display warning after closing browser or after a timeout.
                                choices:
                                    - session
                                    - timeout
                            warning-prompt:
                                description:
                                    - Warning prompts in each category or each domain.
                                choices:
                                    - per-domain
                                    - per-category
                    max-quota-timeout:
                        description:
                            - Maximum FortiGuard quota used by single page view in seconds (excludes streams).
                    options:
                        description:
                            - Options for FortiGuard Web Filter.
                        choices:
                            - error-allow
                            - rate-server-ip
                            - connect-request-bypass
                            - ftgd-disable
                    ovrd:
                        description:
                            - Allow web filter profile overrides.
                    quota:
                        description:
                            - FortiGuard traffic quota settings.
                        suboptions:
                            category:
                                description:
                                    - FortiGuard categories to apply quota to (category action must be set to monitor).
                            duration:
                                description:
                                    - Duration of quota.
                            id:
                                description:
                                    - ID number.
                                required: true
                            override-replacemsg:
                                description:
                                    - Override replacement message.
                            type:
                                description:
                                    - Quota type.
                                choices:
                                    - time
                                    - traffic
                            unit:
                                description:
                                    - Traffic quota unit of measurement.
                                choices:
                                    - B
                                    - KB
                                    - MB
                                    - GB
                            value:
                                description:
                                    - Traffic quota value.
                    rate-crl-urls:
                        description:
                            - Enable/disable rating CRL by URL.
                        choices:
                            - disable
                            - enable
                    rate-css-urls:
                        description:
                            - Enable/disable rating CSS by URL.
                        choices:
                            - disable
                            - enable
                    rate-image-urls:
                        description:
                            - Enable/disable rating images by URL.
                        choices:
                            - disable
                            - enable
                    rate-javascript-urls:
                        description:
                            - Enable/disable rating JavaScript by URL.
                        choices:
                            - disable
                            - enable
            https-replacemsg:
                description:
                    - Enable replacement messages for HTTPS.
                choices:
                    - enable
                    - disable
            inspection-mode:
                description:
                    - Web filtering inspection mode.
                choices:
                    - proxy
                    - flow-based
            log-all-url:
                description:
                    - Enable/disable logging all URLs visited.
                choices:
                    - enable
                    - disable
            name:
                description:
                    - Profile name.
                required: true
            options:
                description:
                    - Options.
                choices:
                    - activexfilter
                    - cookiefilter
                    - javafilter
                    - block-invalid-url
                    - jscript
                    - js
                    - vbs
                    - unknown
                    - intrinsic
                    - wf-referer
                    - wf-cookie
                    - per-user-bwl
            override:
                description:
                    - Web Filter override settings.
                suboptions:
                    ovrd-cookie:
                        description:
                            - Allow/deny browser-based (cookie) overrides.
                        choices:
                            - allow
                            - deny
                    ovrd-dur:
                        description:
                            - Override duration.
                    ovrd-dur-mode:
                        description:
                            - Override duration mode.
                        choices:
                            - constant
                            - ask
                    ovrd-scope:
                        description:
                            - Override scope.
                        choices:
                            - user
                            - user-group
                            - ip
                            - browser
                            - ask
                    ovrd-user-group:
                        description:
                            - User groups with permission to use the override.
                        suboptions:
                            name:
                                description:
                                    - User group name. Source user.group.name.
                                required: true
                    profile:
                        description:
                            - Web filter profile with permission to create overrides.
                        suboptions:
                            name:
                                description:
                                    - Web profile. Source webfilter.profile.name.
                                required: true
                    profile-attribute:
                        description:
                            - Profile attribute to retrieve from the RADIUS server.
                        choices:
                            - User-Name
                            - NAS-IP-Address
                            - Framed-IP-Address
                            - Framed-IP-Netmask
                            - Filter-Id
                            - Login-IP-Host
                            - Reply-Message
                            - Callback-Number
                            - Callback-Id
                            - Framed-Route
                            - Framed-IPX-Network
                            - Class
                            - Called-Station-Id
                            - Calling-Station-Id
                            - NAS-Identifier
                            - Proxy-State
                            - Login-LAT-Service
                            - Login-LAT-Node
                            - Login-LAT-Group
                            - Framed-AppleTalk-Zone
                            - Acct-Session-Id
                            - Acct-Multi-Session-Id
                    profile-type:
                        description:
                            - Override profile type.
                        choices:
                            - list
                            - radius
            ovrd-perm:
                description:
                    - Permitted override types.
                choices:
                    - bannedword-override
                    - urlfilter-override
                    - fortiguard-wf-override
                    - contenttype-check-override
            post-action:
                description:
                    - Action taken for HTTP POST traffic.
                choices:
                    - normal
                    - block
            replacemsg-group:
                description:
                    - Replacement message group. Source system.replacemsg-group.name.
            web:
                description:
                    - Web content filtering settings.
                suboptions:
                    blacklist:
                        description:
                            - Enable/disable automatic addition of URLs detected by FortiSandbox to blacklist.
                        choices:
                            - enable
                            - disable
                    bword-table:
                        description:
                            - Banned word table ID. Source webfilter.content.id.
                    bword-threshold:
                        description:
                            - Banned word score threshold.
                    content-header-list:
                        description:
                            - Content header list. Source webfilter.content-header.id.
                    keyword-match:
                        description:
                            - Search keywords to log when match is found.
                        suboptions:
                            pattern:
                                description:
                                    - Pattern/keyword to search for.
                                required: true
                    log-search:
                        description:
                            - Enable/disable logging all search phrases.
                        choices:
                            - enable
                            - disable
                    safe-search:
                        description:
                            - Safe search type.
                        choices:
                            - url
                            - header
                    urlfilter-table:
                        description:
                            - URL filter table ID. Source webfilter.urlfilter.id.
                    whitelist:
                        description:
                            - FortiGuard whitelist settings.
                        choices:
                            - exempt-av
                            - exempt-webcontent
                            - exempt-activex-java-cookie
                            - exempt-dlp
                            - exempt-rangeblock
                            - extended-log-others
                    youtube-restrict:
                        description:
                            - YouTube EDU filter level.
                        choices:
                            - none
                            - strict
                            - moderate
            web-content-log:
                description:
                    - Enable/disable logging logging blocked web content.
                choices:
                    - enable
                    - disable
            web-extended-all-action-log:
                description:
                    - Enable/disable extended any filter action logging for web filtering.
                choices:
                    - enable
                    - disable
            web-filter-activex-log:
                description:
                    - Enable/disable logging ActiveX.
                choices:
                    - enable
                    - disable
            web-filter-applet-log:
                description:
                    - Enable/disable logging Java applets.
                choices:
                    - enable
                    - disable
            web-filter-command-block-log:
                description:
                    - Enable/disable logging blocked commands.
                choices:
                    - enable
                    - disable
            web-filter-cookie-log:
                description:
                    - Enable/disable logging cookie filtering.
                choices:
                    - enable
                    - disable
            web-filter-cookie-removal-log:
                description:
                    - Enable/disable logging blocked cookies.
                choices:
                    - enable
                    - disable
            web-filter-js-log:
                description:
                    - Enable/disable logging Java scripts.
                choices:
                    - enable
                    - disable
            web-filter-jscript-log:
                description:
                    - Enable/disable logging JScripts.
                choices:
                    - enable
                    - disable
            web-filter-referer-log:
                description:
                    - Enable/disable logging referrers.
                choices:
                    - enable
                    - disable
            web-filter-unknown-log:
                description:
                    - Enable/disable logging unknown scripts.
                choices:
                    - enable
                    - disable
            web-filter-vbs-log:
                description:
                    - Enable/disable logging VBS scripts.
                choices:
                    - enable
                    - disable
            web-ftgd-err-log:
                description:
                    - Enable/disable logging rating errors.
                choices:
                    - enable
                    - disable
            web-ftgd-quota-usage:
                description:
                    - Enable/disable logging daily quota usage.
                choices:
                    - enable
                    - disable
            web-invalid-domain-log:
                description:
                    - Enable/disable logging invalid domain names.
                choices:
                    - enable
                    - disable
            web-url-log:
                description:
                    - Enable/disable logging URL filtering.
                choices:
                    - enable
                    - disable
            wisp:
                description:
                    - Enable/disable web proxy WISP.
                choices:
                    - enable
                    - disable
            wisp-algorithm:
                description:
                    - WISP server selection algorithm.
                choices:
                    - primary-secondary
                    - round-robin
                    - auto-learning
            wisp-servers:
                description:
                    - WISP servers.
                suboptions:
                    name:
                        description:
                            - Server name. Source web-proxy.wisp.name.
                        required: true
            youtube-channel-filter:
                description:
                    - YouTube channel filter.
                suboptions:
                    channel-id:
                        description:
                            - YouTube channel ID to be filtered.
                    comment:
                        description:
                            - Comment.
                    id:
                        description:
                            - ID.
                        required: true
            youtube-channel-status:
                description:
                    - YouTube channel filter status.
                choices:
                    - disable
                    - blacklist
                    - whitelist
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure Web filter profiles.
    fortios_webfilter_profile:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      webfilter_profile:
        state: "present"
        comment: "Optional comments."
        extended-log: "enable"
        ftgd-wf:
            exempt-quota: "<your_own_value>"
            filters:
             -
                action: "block"
                auth-usr-grp:
                 -
                    name: "default_name_10 (source user.group.name)"
                category: "11"
                id:  "12"
                log: "enable"
                override-replacemsg: "<your_own_value>"
                warn-duration: "<your_own_value>"
                warning-duration-type: "session"
                warning-prompt: "per-domain"
            max-quota-timeout: "18"
            options: "error-allow"
            ovrd: "<your_own_value>"
            quota:
             -
                category: "<your_own_value>"
                duration: "<your_own_value>"
                id:  "24"
                override-replacemsg: "<your_own_value>"
                type: "time"
                unit: "B"
                value: "28"
            rate-crl-urls: "disable"
            rate-css-urls: "disable"
            rate-image-urls: "disable"
            rate-javascript-urls: "disable"
        https-replacemsg: "enable"
        inspection-mode: "proxy"
        log-all-url: "enable"
        name: "default_name_36"
        options: "activexfilter"
        override:
            ovrd-cookie: "allow"
            ovrd-dur: "<your_own_value>"
            ovrd-dur-mode: "constant"
            ovrd-scope: "user"
            ovrd-user-group:
             -
                name: "default_name_44 (source user.group.name)"
            profile:
             -
                name: "default_name_46 (source webfilter.profile.name)"
            profile-attribute: "User-Name"
            profile-type: "list"
        ovrd-perm: "bannedword-override"
        post-action: "normal"
        replacemsg-group: "<your_own_value> (source system.replacemsg-group.name)"
        web:
            blacklist: "enable"
            bword-table: "54 (source webfilter.content.id)"
            bword-threshold: "55"
            content-header-list: "56 (source webfilter.content-header.id)"
            keyword-match:
             -
                pattern: "<your_own_value>"
            log-search: "enable"
            safe-search: "url"
            urlfilter-table: "61 (source webfilter.urlfilter.id)"
            whitelist: "exempt-av"
            youtube-restrict: "none"
        web-content-log: "enable"
        web-extended-all-action-log: "enable"
        web-filter-activex-log: "enable"
        web-filter-applet-log: "enable"
        web-filter-command-block-log: "enable"
        web-filter-cookie-log: "enable"
        web-filter-cookie-removal-log: "enable"
        web-filter-js-log: "enable"
        web-filter-jscript-log: "enable"
        web-filter-referer-log: "enable"
        web-filter-unknown-log: "enable"
        web-filter-vbs-log: "enable"
        web-ftgd-err-log: "enable"
        web-ftgd-quota-usage: "enable"
        web-invalid-domain-log: "enable"
        web-url-log: "enable"
        wisp: "enable"
        wisp-algorithm: "primary-secondary"
        wisp-servers:
         -
            name: "default_name_83 (source web-proxy.wisp.name)"
        youtube-channel-filter:
         -
            channel-id: "<your_own_value>"
            comment: "Comment."
            id:  "87"
        youtube-channel-status: "disable"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "key1"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_webfilter_profile_data(json):
    option_list = ['comment', 'extended-log', 'ftgd-wf',
                   'https-replacemsg', 'inspection-mode', 'log-all-url',
                   'name', 'options', 'override',
                   'ovrd-perm', 'post-action', 'replacemsg-group',
                   'web', 'web-content-log', 'web-extended-all-action-log',
                   'web-filter-activex-log', 'web-filter-applet-log', 'web-filter-command-block-log',
                   'web-filter-cookie-log', 'web-filter-cookie-removal-log', 'web-filter-js-log',
                   'web-filter-jscript-log', 'web-filter-referer-log', 'web-filter-unknown-log',
                   'web-filter-vbs-log', 'web-ftgd-err-log', 'web-ftgd-quota-usage',
                   'web-invalid-domain-log', 'web-url-log', 'wisp',
                   'wisp-algorithm', 'wisp-servers', 'youtube-channel-filter',
                   'youtube-channel-status']
    dictionary = {}

    for attribute in option_list:
        if attribute in json:
            dictionary[attribute] = json[attribute]

    return dictionary


def webfilter_profile(data, fos):
    vdom = data['vdom']
    webfilter_profile_data = data['webfilter_profile']
    filtered_data = filter_webfilter_profile_data(webfilter_profile_data)
    if webfilter_profile_data['state'] == "present":
        return fos.set('webfilter',
                       'profile',
                       data=filtered_data,
                       vdom=vdom)

    elif webfilter_profile_data['state'] == "absent":
        return fos.delete('webfilter',
                          'profile',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_webfilter(data, fos):
    login(data)

    methodlist = ['webfilter_profile']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "webfilter_profile": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "comment": {"required": False, "type": "str"},
                "extended-log": {"required": False, "type": "str",
                                 "choices": ["enable", "disable"]},
                "ftgd-wf": {"required": False, "type": "dict",
                            "options": {
                                "exempt-quota": {"required": False, "type": "str"},
                                "filters": {"required": False, "type": "list",
                                            "options": {
                                                "action": {"required": False, "type": "str",
                                                           "choices": ["block", "authenticate", "monitor",
                                                                       "warning"]},
                                                "auth-usr-grp": {"required": False, "type": "str",
                                                                 "options": {
                                                                     "name": {"required": True, "type": "str"}
                                                                 }},
                                                "category": {"required": False, "type": "int"},
                                                "id": {"required": True, "type": "int"},
                                                "log": {"required": False, "type": "str",
                                                        "choices": ["enable", "disable"]},
                                                "override-replacemsg": {"required": False, "type": "str"},
                                                "warn-duration": {"required": False, "type": "str"},
                                                "warning-duration-type": {"required": False, "type": "str",
                                                                          "choices": ["session", "timeout"]},
                                                "warning-prompt": {"required": False, "type": "str",
                                                                   "choices": ["per-domain", "per-category"]}
                                            }},
                                "max-quota-timeout": {"required": False, "type": "int"},
                                "options": {"required": False, "type": "str",
                                            "choices": ["error-allow", "rate-server-ip", "connect-request-bypass",
                                                        "ftgd-disable"]},
                                "ovrd": {"required": False, "type": "str"},
                                "quota": {"required": False, "type": "list",
                                          "options": {
                                              "category": {"required": False, "type": "str"},
                                              "duration": {"required": False, "type": "str"},
                                              "id": {"required": True, "type": "int"},
                                              "override-replacemsg": {"required": False, "type": "str"},
                                              "type": {"required": False, "type": "str",
                                                       "choices": ["time", "traffic"]},
                                              "unit": {"required": False, "type": "str",
                                                       "choices": ["B", "KB", "MB",
                                                                   "GB"]},
                                              "value": {"required": False, "type": "int"}
                                          }},
                                "rate-crl-urls": {"required": False, "type": "str",
                                                  "choices": ["disable", "enable"]},
                                "rate-css-urls": {"required": False, "type": "str",
                                                  "choices": ["disable", "enable"]},
                                "rate-image-urls": {"required": False, "type": "str",
                                                    "choices": ["disable", "enable"]},
                                "rate-javascript-urls": {"required": False, "type": "str",
                                                         "choices": ["disable", "enable"]}
                            }},
                "https-replacemsg": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "inspection-mode": {"required": False, "type": "str",
                                    "choices": ["proxy", "flow-based"]},
                "log-all-url": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "name": {"required": True, "type": "str"},
                "options": {"required": False, "type": "str",
                            "choices": ["activexfilter", "cookiefilter", "javafilter",
                                        "block-invalid-url", "jscript", "js",
                                        "vbs", "unknown", "intrinsic",
                                        "wf-referer", "wf-cookie", "per-user-bwl"]},
                "override": {"required": False, "type": "dict",
                             "options": {
                                 "ovrd-cookie": {"required": False, "type": "str",
                                                 "choices": ["allow", "deny"]},
                                 "ovrd-dur": {"required": False, "type": "str"},
                                 "ovrd-dur-mode": {"required": False, "type": "str",
                                                   "choices": ["constant", "ask"]},
                                 "ovrd-scope": {"required": False, "type": "str",
                                                "choices": ["user", "user-group", "ip",
                                                            "browser", "ask"]},
                                 "ovrd-user-group": {"required": False, "type": "str",
                                                     "options": {
                                                         "name": {"required": True, "type": "str"}
                                                     }},
                                 "profile": {"required": False, "type": "list",
                                             "options": {
                                                 "name": {"required": True, "type": "str"}
                                             }},
                                 "profile-attribute": {"required": False, "type": "str",
                                                       "choices": ["User-Name", "NAS-IP-Address", "Framed-IP-Address",
                                                                   "Framed-IP-Netmask", "Filter-Id", "Login-IP-Host",
                                                                   "Reply-Message", "Callback-Number", "Callback-Id",
                                                                   "Framed-Route", "Framed-IPX-Network", "Class",
                                                                   "Called-Station-Id", "Calling-Station-Id", "NAS-Identifier",
                                                                   "Proxy-State", "Login-LAT-Service", "Login-LAT-Node",
                                                                   "Login-LAT-Group", "Framed-AppleTalk-Zone", "Acct-Session-Id",
                                                                   "Acct-Multi-Session-Id"]},
                                 "profile-type": {"required": False, "type": "str",
                                                  "choices": ["list", "radius"]}
                             }},
                "ovrd-perm": {"required": False, "type": "str",
                              "choices": ["bannedword-override", "urlfilter-override", "fortiguard-wf-override",
                                          "contenttype-check-override"]},
                "post-action": {"required": False, "type": "str",
                                "choices": ["normal", "block"]},
                "replacemsg-group": {"required": False, "type": "str"},
                "web": {"required": False, "type": "dict",
                        "options": {
                            "blacklist": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                            "bword-table": {"required": False, "type": "int"},
                            "bword-threshold": {"required": False, "type": "int"},
                            "content-header-list": {"required": False, "type": "int"},
                            "keyword-match": {"required": False, "type": "str",
                                              "options": {
                                                  "pattern": {"required": True, "type": "str"}
                                              }},
                            "log-search": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                            "safe-search": {"required": False, "type": "str",
                                            "choices": ["url", "header"]},
                            "urlfilter-table": {"required": False, "type": "int"},
                            "whitelist": {"required": False, "type": "str",
                                          "choices": ["exempt-av", "exempt-webcontent", "exempt-activex-java-cookie",
                                                      "exempt-dlp", "exempt-rangeblock", "extended-log-others"]},
                            "youtube-restrict": {"required": False, "type": "str",
                                                 "choices": ["none", "strict", "moderate"]}
                        }},
                "web-content-log": {"required": False, "type": "str",
                                    "choices": ["enable", "disable"]},
                "web-extended-all-action-log": {"required": False, "type": "str",
                                                "choices": ["enable", "disable"]},
                "web-filter-activex-log": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "web-filter-applet-log": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "web-filter-command-block-log": {"required": False, "type": "str",
                                                 "choices": ["enable", "disable"]},
                "web-filter-cookie-log": {"required": False, "type": "str",
                                          "choices": ["enable", "disable"]},
                "web-filter-cookie-removal-log": {"required": False, "type": "str",
                                                  "choices": ["enable", "disable"]},
                "web-filter-js-log": {"required": False, "type": "str",
                                      "choices": ["enable", "disable"]},
                "web-filter-jscript-log": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "web-filter-referer-log": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "web-filter-unknown-log": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "web-filter-vbs-log": {"required": False, "type": "str",
                                       "choices": ["enable", "disable"]},
                "web-ftgd-err-log": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "web-ftgd-quota-usage": {"required": False, "type": "str",
                                         "choices": ["enable", "disable"]},
                "web-invalid-domain-log": {"required": False, "type": "str",
                                           "choices": ["enable", "disable"]},
                "web-url-log": {"required": False, "type": "str",
                                "choices": ["enable", "disable"]},
                "wisp": {"required": False, "type": "str",
                         "choices": ["enable", "disable"]},
                "wisp-algorithm": {"required": False, "type": "str",
                                   "choices": ["primary-secondary", "round-robin", "auto-learning"]},
                "wisp-servers": {"required": False, "type": "list",
                                 "options": {
                                     "name": {"required": True, "type": "str"}
                                 }},
                "youtube-channel-filter": {"required": False, "type": "list",
                                           "options": {
                                               "channel-id": {"required": False, "type": "str"},
                                               "comment": {"required": False, "type": "str"},
                                               "id": {"required": True, "type": "int"}
                                           }},
                "youtube-channel-status": {"required": False, "type": "str",
                                           "choices": ["disable", "blacklist", "whitelist"]}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_webfilter(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



# (c) 2016 Red Hat Inc.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

from units.compat.mock import patch
from ansible.modules.network.vyos import vyos_ping
from units.modules.utils import set_module_args
from .vyos_module import TestVyosModule, load_fixture


class TestVyosPingModule(TestVyosModule):

    module = vyos_ping

    def setUp(self):
        super(TestVyosPingModule, self).setUp()
        self.mock_run_commands = patch('ansible.modules.network.vyos.vyos_ping.run_commands')
        self.run_commands = self.mock_run_commands.start()

    def tearDown(self):
        super(TestVyosPingModule, self).tearDown()
        self.mock_run_commands.stop()

    def load_fixtures(self, commands=None):
        def load_from_file(*args, **kwargs):
            commands = kwargs['commands']
            output = list()

            for command in commands:
                filename = str(command).split(' | ')[0].replace(' ', '_')
                output.append(load_fixture('vyos_ping_%s' % filename))
            return output

        self.run_commands.side_effect = load_from_file

    def test_vyos_ping_expected_success(self):
        ''' Test for successful pings when destination should be reachable '''
        set_module_args(dict(count=2, dest="10.10.10.10"))
        self.execute_module()

    def test_vyos_ping_expected_failure(self):
        ''' Test for unsuccessful pings when destination should not be reachable '''
        set_module_args(dict(count=4, dest="10.10.10.20", state="absent"))
        self.execute_module()

    def test_vyos_ping_unexpected_success(self):
        ''' Test for successful pings when destination should not be reachable - FAIL. '''
        set_module_args(dict(count=2, dest="10.10.10.10", state="absent"))
        self.execute_module(failed=True)

    def test_vyos_ping_unexpected_failure(self):
        ''' Test for unsuccessful pings when destination should be reachable - FAIL. '''
        set_module_args(dict(count=4, dest="10.10.10.20"))
        self.execute_module(failed=True)

    def test_vyos_ping_failure_stats(self):
        '''Test for asserting stats when ping fails'''
        set_module_args(dict(count=4, dest="10.10.10.20"))
        result = self.execute_module(failed=True)
        self.assertEqual(result['packet_loss'], '100%')
        self.assertEqual(result['packets_rx'], 0)
        self.assertEqual(result['packets_tx'], 4)

    def test_vyos_ping_success_stats(self):
        '''Test for asserting stats when ping passes'''
        set_module_args(dict(count=2, dest="10.10.10.10"))
        result = self.execute_module()
        self.assertEqual(result['packet_loss'], '0%')
        self.assertEqual(result['packets_rx'], 2)
        self.assertEqual(result['packets_tx'], 2)
        self.assertEqual(result['rtt']['min'], 12)
        self.assertEqual(result['rtt']['avg'], 17)
        self.assertEqual(result['rtt']['max'], 22)
        self.assertEqual(result['rtt']['mdev'], 10)

    def test_vyos_ping_success_stats_with_options(self):
        set_module_args(dict(count=10, ttl=128, size=512, dest="10.10.10.11"))
        result = self.execute_module()
        self.assertEqual(result['packet_loss'], '0%')
        self.assertEqual(result['packets_rx'], 10)
        self.assertEqual(result['packets_tx'], 10)
        self.assertEqual(result['rtt']['min'], 1)
        self.assertEqual(result['rtt']['avg'], 3)
        self.assertEqual(result['rtt']['max'], 21)
        self.assertEqual(result['rtt']['mdev'], 5)



# This code is part of Ansible, but is an independent component.
# This particular file snippet, and this file snippet only, is BSD licensed.
# Modules you write using this snippet, which is embedded dynamically by Ansible
# still belong to the author of the module, and may assign their own license
# to the complete work.
#
# Copyright (c) 2017, Sumit Kumar <sumit4@netapp.com>
# Copyright (c) 2017, Michael Price <michael.price@netapp.com>
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#    * Redistributions in binary form must reproduce the above copyright notice,
#      this list of conditions and the following disclaimer in the documentation
#      and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
# USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import json
import os

from pprint import pformat
from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.six.moves.urllib.error import HTTPError, URLError
from ansible.module_utils.urls import open_url
from ansible.module_utils.api import basic_auth_argument_spec
from ansible.module_utils._text import to_native

try:
    from ansible.module_utils.ansible_release import __version__ as ansible_version
except ImportError:
    ansible_version = 'unknown'

try:
    from netapp_lib.api.zapi import zapi
    HAS_NETAPP_LIB = True
except ImportError:
    HAS_NETAPP_LIB = False

import ssl
try:
    from urlparse import urlparse, urlunparse
except ImportError:
    from urllib.parse import urlparse, urlunparse


HAS_SF_SDK = False
SF_BYTE_MAP = dict(
    # Management GUI displays 1024 ** 3 as 1.1 GB, thus use 1000.
    bytes=1,
    b=1,
    kb=1000,
    mb=1000 ** 2,
    gb=1000 ** 3,
    tb=1000 ** 4,
    pb=1000 ** 5,
    eb=1000 ** 6,
    zb=1000 ** 7,
    yb=1000 ** 8
)

POW2_BYTE_MAP = dict(
    # Here, 1 kb = 1024
    bytes=1,
    b=1,
    kb=1024,
    mb=1024 ** 2,
    gb=1024 ** 3,
    tb=1024 ** 4,
    pb=1024 ** 5,
    eb=1024 ** 6,
    zb=1024 ** 7,
    yb=1024 ** 8
)

try:
    from solidfire.factory import ElementFactory
    from solidfire.custom.models import TimeIntervalFrequency
    from solidfire.models import Schedule, ScheduleInfo

    HAS_SF_SDK = True
except Exception:
    HAS_SF_SDK = False


def has_netapp_lib():
    return HAS_NETAPP_LIB


def has_sf_sdk():
    return HAS_SF_SDK


def na_ontap_host_argument_spec():

    return dict(
        hostname=dict(required=True, type='str'),
        username=dict(required=True, type='str', aliases=['user']),
        password=dict(required=True, type='str', aliases=['pass'], no_log=True),
        https=dict(required=False, type='bool', default=False),
        validate_certs=dict(required=False, type='bool', default=True),
        http_port=dict(required=False, type='int'),
        ontapi=dict(required=False, type='int')
    )


def ontap_sf_host_argument_spec():

    return dict(
        hostname=dict(required=True, type='str'),
        username=dict(required=True, type='str', aliases=['user']),
        password=dict(required=True, type='str', aliases=['pass'], no_log=True)
    )


def create_sf_connection(module, port=None):
    hostname = module.params['hostname']
    username = module.params['username']
    password = module.params['password']

    if HAS_SF_SDK and hostname and username and password:
        try:
            return_val = ElementFactory.create(hostname, username, password, port=port)
            return return_val
        except Exception:
            raise Exception("Unable to create SF connection")
    else:
        module.fail_json(msg="the python SolidFire SDK module is required")


def setup_na_ontap_zapi(module, vserver=None):
    hostname = module.params['hostname']
    username = module.params['username']
    password = module.params['password']
    https = module.params['https']
    validate_certs = module.params['validate_certs']
    port = module.params['http_port']
    version = module.params['ontapi']

    if HAS_NETAPP_LIB:
        # set up zapi
        server = zapi.NaServer(hostname)
        server.set_username(username)
        server.set_password(password)
        if vserver:
            server.set_vserver(vserver)
        if version:
            minor = version
        else:
            minor = 110
        server.set_api_version(major=1, minor=minor)
        # default is HTTP
        if https:
            if port is None:
                port = 443
            transport_type = 'HTTPS'
            # HACK to bypass certificate verification
            if validate_certs is False:
                if not os.environ.get('PYTHONHTTPSVERIFY', '') and getattr(ssl, '_create_unverified_context', None):
                    ssl._create_default_https_context = ssl._create_unverified_context
        else:
            if port is None:
                port = 80
            transport_type = 'HTTP'
        server.set_transport_type(transport_type)
        server.set_port(port)
        server.set_server_type('FILER')
        return server
    else:
        module.fail_json(msg="the python NetApp-Lib module is required")


def setup_ontap_zapi(module, vserver=None):
    hostname = module.params['hostname']
    username = module.params['username']
    password = module.params['password']

    if HAS_NETAPP_LIB:
        # set up zapi
        server = zapi.NaServer(hostname)
        server.set_username(username)
        server.set_password(password)
        if vserver:
            server.set_vserver(vserver)
        # Todo : Replace hard-coded values with configurable parameters.
        server.set_api_version(major=1, minor=110)
        server.set_port(80)
        server.set_server_type('FILER')
        server.set_transport_type('HTTP')
        return server
    else:
        module.fail_json(msg="the python NetApp-Lib module is required")


def eseries_host_argument_spec():
    """Retrieve a base argument specification common to all NetApp E-Series modules"""
    argument_spec = basic_auth_argument_spec()
    argument_spec.update(dict(
        api_username=dict(type='str', required=True),
        api_password=dict(type='str', required=True, no_log=True),
        api_url=dict(type='str', required=True),
        ssid=dict(type='str', required=False, default='1'),
        validate_certs=dict(type='bool', required=False, default=True)
    ))
    return argument_spec


class NetAppESeriesModule(object):
    """Base class for all NetApp E-Series modules.

    Provides a set of common methods for NetApp E-Series modules, including version checking, mode (proxy, embedded)
    verification, http requests, secure http redirection for embedded web services, and logging setup.

    Be sure to add the following lines in the module's documentation section:
    extends_documentation_fragment:
        - netapp.eseries

    :param dict(dict) ansible_options: dictionary of ansible option definitions
    :param str web_services_version: minimally required web services rest api version (default value: "02.00.0000.0000")
    :param bool supports_check_mode: whether the module will support the check_mode capabilities (default=False)
    :param list(list) mutually_exclusive: list containing list(s) of mutually exclusive options (optional)
    :param list(list) required_if: list containing list(s) containing the option, the option value, and then
    a list of required options. (optional)
    :param list(list) required_one_of: list containing list(s) of options for which at least one is required. (optional)
    :param list(list) required_together: list containing list(s) of options that are required together. (optional)
    :param bool log_requests: controls whether to log each request (default: True)
    """
    DEFAULT_TIMEOUT = 60
    DEFAULT_SECURE_PORT = "8443"
    DEFAULT_REST_API_PATH = "devmgr/v2"
    DEFAULT_HEADERS = {"Content-Type": "application/json", "Accept": "application/json",
                       "netapp-client-type": "Ansible-%s" % ansible_version}
    HTTP_AGENT = "Ansible / %s" % ansible_version
    SIZE_UNIT_MAP = dict(bytes=1, b=1, kb=1024, mb=1024**2, gb=1024**3, tb=1024**4,
                         pb=1024**5, eb=1024**6, zb=1024**7, yb=1024**8)

    def __init__(self, ansible_options, web_services_version=None, supports_check_mode=False,
                 mutually_exclusive=None, required_if=None, required_one_of=None, required_together=None,
                 log_requests=True):
        argument_spec = eseries_host_argument_spec()
        argument_spec.update(ansible_options)

        self.module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=supports_check_mode,
                                    mutually_exclusive=mutually_exclusive, required_if=required_if,
                                    required_one_of=required_one_of, required_together=required_together)

        args = self.module.params
        self.web_services_version = web_services_version if web_services_version else "02.00.0000.0000"
        self.url = args["api_url"]
        self.ssid = args["ssid"]
        self.creds = dict(url_username=args["api_username"],
                          url_password=args["api_password"],
                          validate_certs=args["validate_certs"])
        self.log_requests = log_requests

        self.is_embedded_mode = None
        self.web_services_validate = None

        self._tweak_url()

    @property
    def _about_url(self):
        """Generates the about url based on the supplied web services rest api url.

        :raise AnsibleFailJson: raised when supplied web services rest api is an invalid url.
        :return: proxy or embedded about url.
        """
        about = list(urlparse(self.url))
        about[2] = "devmgr/utils/about"
        return urlunparse(about)

    def _force_secure_url(self):
        """Modifies supplied web services rest api to use secure https.

        raise: AnsibleFailJson: raised when the url already utilizes the secure protocol
        """
        url_parts = list(urlparse(self.url))
        if "https://" in self.url and ":8443" in self.url:
            self.module.fail_json(msg="Secure HTTP protocol already used. URL path [%s]" % self.url)

        url_parts[0] = "https"
        url_parts[1] = "%s:8443" % url_parts[1].split(":")[0]

        self.url = urlunparse(url_parts)
        if not self.url.endswith("/"):
            self.url += "/"

        self.module.warn("forced use of the secure protocol: %s" % self.url)

    def _tweak_url(self):
        """Adjust the rest api url is necessary.

        :raise AnsibleFailJson: raised when self.url fails to have a hostname or ipv4 address.
        """
        # ensure the protocol is either http or https
        if self.url.split("://")[0] not in ["https", "http"]:

            self.url = self.url.split("://")[1] if "://" in self.url else self.url

            if ":8080" in self.url:
                self.url = "http://%s" % self.url
            else:
                self.url = "https://%s" % self.url

        # parse url and verify protocol, port and path are consistent with required web services rest api url.
        url_parts = list(urlparse(self.url))
        if url_parts[1] == "":
            self.module.fail_json(msg="Failed to provide a valid hostname or IP address. URL [%s]." % self.url)

        split_hostname = url_parts[1].split(":")
        if url_parts[0] not in ["https", "http"] or (len(split_hostname) == 2 and split_hostname[1] != "8080"):
            if len(split_hostname) == 2 and split_hostname[1] == "8080":
                url_parts[0] = "http"
                url_parts[1] = "%s:8080" % split_hostname[0]
            else:
                url_parts[0] = "https"
                url_parts[1] = "%s:8443" % split_hostname[0]
        elif len(split_hostname) == 1:
            if url_parts[0] == "https":
                url_parts[1] = "%s:8443" % split_hostname[0]
            elif url_parts[0] == "http":
                url_parts[1] = "%s:8080" % split_hostname[0]

        if url_parts[2] == "" or url_parts[2] != self.DEFAULT_REST_API_PATH:
            url_parts[2] = self.DEFAULT_REST_API_PATH

        self.url = urlunparse(url_parts)
        if not self.url.endswith("/"):
            self.url += "/"

        self.module.log("valid url: %s" % self.url)

    def _is_web_services_valid(self):
        """Verify proxy or embedded web services meets minimum version required for module.

        The minimum required web services version is evaluated against version supplied through the web services rest
        api. AnsibleFailJson exception will be raised when the minimum is not met or exceeded.

        :raise AnsibleFailJson: raised when the contacted api service does not meet the minimum required version.
        """
        if not self.web_services_validate:
            self.is_embedded()
            try:
                rc, data = request(self._about_url, timeout=self.DEFAULT_TIMEOUT,
                                   headers=self.DEFAULT_HEADERS, **self.creds)
                major, minor, other, revision = data["version"].split(".")
                minimum_major, minimum_minor, other, minimum_revision = self.web_services_version.split(".")

                if not (major > minimum_major or
                        (major == minimum_major and minor > minimum_minor) or
                        (major == minimum_major and minor == minimum_minor and revision >= minimum_revision)):
                    self.module.fail_json(
                        msg="Web services version does not meet minimum version required. Current version: [%s]."
                            " Version required: [%s]." % (data["version"], self.web_services_version))
            except Exception as error:
                self.module.fail_json(msg="Failed to retrieve the webservices about information! Array Id [%s]."
                                          " Error [%s]." % (self.ssid, to_native(error)))

            self.module.warn("Web services rest api version met the minimum required version.")
            self.web_services_validate = True

        return self.web_services_validate

    def is_embedded(self, retry=True):
        """Determine whether web services server is the embedded web services.

        If web services about endpoint fails based on an URLError then the request will be attempted again using
        secure http.

        :raise AnsibleFailJson: raised when web services about endpoint failed to be contacted.
        :return bool: whether contacted web services is running from storage array (embedded) or from a proxy.
        """
        if self.is_embedded_mode is None:
            try:
                rc, data = request(self._about_url, timeout=self.DEFAULT_TIMEOUT,
                                   headers=self.DEFAULT_HEADERS, **self.creds)
                self.is_embedded_mode = not data["runningAsProxy"]
            except URLError as error:
                if not retry:
                    self.module.fail_json(msg="Failed to retrieve the webservices about information! Array Id [%s]."
                                              " Error [%s]." % (self.ssid, to_native(error)))
                self.module.warn("Failed to retrieve the webservices about information! Will retry using secure"
                                 " http. Array Id [%s]." % self.ssid)
                self._force_secure_url()
                return self.is_embedded(retry=False)
            except Exception as error:
                self.module.fail_json(msg="Failed to retrieve the webservices about information! Array Id [%s]."
                                          " Error [%s]." % (self.ssid, to_native(error)))

        return self.is_embedded_mode

    def request(self, path, data=None, method='GET', ignore_errors=False):
        """Issue an HTTP request to a url, retrieving an optional JSON response.

        :param str path: web services rest api endpoint path (Example: storage-systems/1/graph). Note that when the
        full url path is specified then that will be used without supplying the protocol, hostname, port and rest path.
        :param data: data required for the request (data may be json or any python structured data)
        :param str method: request method such as GET, POST, DELETE.
        :param bool ignore_errors: forces the request to ignore any raised exceptions.
        """
        if self._is_web_services_valid():
            url = list(urlparse(path.strip("/")))
            if url[2] == "":
                self.module.fail_json(msg="Web services rest api endpoint path must be specified. Path [%s]." % path)

            # if either the protocol or hostname/port are missing then add them.
            if url[0] == "" or url[1] == "":
                url[0], url[1] = list(urlparse(self.url))[:2]

            # add rest api path if the supplied path does not begin with it.
            if not all([word in url[2].split("/")[:2] for word in self.DEFAULT_REST_API_PATH.split("/")]):
                if not url[2].startswith("/"):
                    url[2] = "/" + url[2]
                url[2] = self.DEFAULT_REST_API_PATH + url[2]

            # ensure data is json formatted
            if not isinstance(data, str):
                data = json.dumps(data)

            if self.log_requests:
                self.module.log(pformat(dict(url=urlunparse(url), data=data, method=method)))

            return request(url=urlunparse(url), data=data, method=method, headers=self.DEFAULT_HEADERS, use_proxy=True,
                           force=False, last_mod_time=None, timeout=self.DEFAULT_TIMEOUT, http_agent=self.HTTP_AGENT,
                           force_basic_auth=True, ignore_errors=ignore_errors, **self.creds)


def request(url, data=None, headers=None, method='GET', use_proxy=True,
            force=False, last_mod_time=None, timeout=10, validate_certs=True,
            url_username=None, url_password=None, http_agent=None, force_basic_auth=True, ignore_errors=False):
    """Issue an HTTP request to a url, retrieving an optional JSON response."""

    if headers is None:
        headers = {"Content-Type": "application/json", "Accept": "application/json"}
    headers.update({"netapp-client-type": "Ansible-%s" % ansible_version})

    if not http_agent:
        http_agent = "Ansible / %s" % ansible_version

    try:
        r = open_url(url=url, data=data, headers=headers, method=method, use_proxy=use_proxy,
                     force=force, last_mod_time=last_mod_time, timeout=timeout, validate_certs=validate_certs,
                     url_username=url_username, url_password=url_password, http_agent=http_agent,
                     force_basic_auth=force_basic_auth)
    except HTTPError as err:
        r = err.fp

    try:
        raw_data = r.read()
        if raw_data:
            data = json.loads(raw_data)
        else:
            raw_data = None
    except Exception:
        if ignore_errors:
            pass
        else:
            raise Exception(raw_data)

    resp_code = r.getcode()

    if resp_code >= 400 and not ignore_errors:
        raise Exception(resp_code, data)
    else:
        return resp_code, data


def ems_log_event(source, server, name="Ansible", id="12345", version=ansible_version,
                  category="Information", event="setup", autosupport="false"):
    ems_log = zapi.NaElement('ems-autosupport-log')
    # Host name invoking the API.
    ems_log.add_new_child("computer-name", name)
    # ID of event. A user defined event-id, range [0..2^32-2].
    ems_log.add_new_child("event-id", id)
    # Name of the application invoking the API.
    ems_log.add_new_child("event-source", source)
    # Version of application invoking the API.
    ems_log.add_new_child("app-version", version)
    # Application defined category of the event.
    ems_log.add_new_child("category", category)
    # Description of event to log. An application defined message to log.
    ems_log.add_new_child("event-description", event)
    ems_log.add_new_child("log-level", "6")
    ems_log.add_new_child("auto-support", autosupport)
    server.invoke_successfully(ems_log, True)


def get_cserver(server):
    vserver_info = zapi.NaElement('vserver-get-iter')
    query_details = zapi.NaElement.create_node_with_children('vserver-info', **{'vserver-type': 'admin'})
    query = zapi.NaElement('query')
    query.add_child_elem(query_details)
    vserver_info.add_child_elem(query)
    result = server.invoke_successfully(vserver_info,
                                        enable_tunneling=False)
    attribute_list = result.get_child_by_name('attributes-list')
    vserver_list = attribute_list.get_child_by_name('vserver-info')
    return vserver_list.get_child_content('vserver-name')



#
# (c) 2018 Lenovo.
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import re
import json

from units.compat.mock import patch
from ansible.modules.network.cnos import cnos_l3_interface
from units.modules.utils import set_module_args
from .cnos_module import TestCnosModule, load_fixture


class TestCnosL3InterfaceModule(TestCnosModule):
    module = cnos_l3_interface

    def setUp(self):
        super(TestCnosL3InterfaceModule, self).setUp()
        self._patch_get_config = patch(
            'ansible.modules.network.cnos.cnos_l3_interface.get_config'
        )
        self._patch_load_config = patch(
            'ansible.modules.network.cnos.cnos_l3_interface.load_config'
        )
        self._patch_is_switchport = patch(
            'ansible.modules.network.cnos.cnos_l3_interface.is_switchport'
        )

        self._get_config = self._patch_get_config.start()
        self._load_config = self._patch_load_config.start()
        self._is_switchport = self._patch_is_switchport.start()

    def tearDown(self):
        super(TestCnosL3InterfaceModule, self).tearDown()
        self._patch_get_config.stop()
        self._patch_load_config.stop()

    def load_fixtures(self, commands=None):
        config_file = 'l3_interface_config.cfg'
        self._get_config.return_value = load_fixture(config_file)
        self._load_config.return_value = None
        self._is_switchport.return_value = False

    def test_cnos_l3_interface_ipv4_address(self, *args, **kwargs):
        set_module_args(dict(
            name='Ethernet 1/35',
            ipv4='192.168.4.1/24'
        ))
        commands = [
            'interface Ethernet 1/35',
            'ip address 192.168.4.1 255.255.255.0'
        ]
        result = self.execute_module(changed=True, commands=commands)

    def test_cnos_l3_interface_absent(self, *args, **kwargs):
        set_module_args(dict(
            name='Ethernet1/9',
            state='absent'
        ))
        commands = [
            'interface Ethernet1/9',
            'no ip address',
            'no ipv6 address'
        ]
        result = self.execute_module(changed=True, commands=commands)



#!/usr/bin/python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2017 Google
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)
# ----------------------------------------------------------------------------
#
#     ***     AUTO GENERATED CODE    ***    AUTO GENERATED CODE     ***
#
# ----------------------------------------------------------------------------
#
#     This file is automatically generated by Magic Modules and manual
#     changes will be clobbered when the file is regenerated.
#
#     Please read more about how to change this file at
#     https://www.github.com/GoogleCloudPlatform/magic-modules
#
# ----------------------------------------------------------------------------

from __future__ import absolute_import, division, print_function

__metaclass__ = type

################################################################################
# Documentation
################################################################################

ANSIBLE_METADATA = {'metadata_version': '1.1', 'status': ["preview"], 'supported_by': 'community'}

DOCUMENTATION = '''
---
module: gcp_compute_router_facts
description:
- Gather facts for GCP Router
short_description: Gather facts for GCP Router
version_added: 2.7
author: Google Inc. (@googlecloudplatform)
requirements:
- python >= 2.6
- requests >= 2.18.4
- google-auth >= 1.3.0
options:
  filters:
    description:
    - A list of filter value pairs. Available filters are listed here U(https://cloud.google.com/sdk/gcloud/reference/topic/filters.)
    - Each additional filter in the list will act be added as an AND condition (filter1
      and filter2) .
  region:
    description:
    - Region where the router resides.
    required: true
extends_documentation_fragment: gcp
'''

EXAMPLES = '''
- name: " a router facts"
  gcp_compute_router_facts:
    region: us-central1
    filters:
    - name = test_object
    project: test_project
    auth_kind: serviceaccount
    service_account_file: "/tmp/auth.pem"
    state: facts
'''

RETURN = '''
items:
  description: List of items
  returned: always
  type: complex
  contains:
    id:
      description:
      - The unique identifier for the resource.
      returned: success
      type: int
    creationTimestamp:
      description:
      - Creation timestamp in RFC3339 text format.
      returned: success
      type: str
    name:
      description:
      - Name of the resource. The name must be 1-63 characters long, and comply with
        RFC1035. Specifically, the name must be 1-63 characters long and match the
        regular expression `[a-z]([-a-z0-9]*[a-z0-9])?` which means the first character
        must be a lowercase letter, and all following characters must be a dash, lowercase
        letter, or digit, except the last character, which cannot be a dash.
      returned: success
      type: str
    description:
      description:
      - An optional description of this resource.
      returned: success
      type: str
    network:
      description:
      - A reference to the network to which this router belongs.
      returned: success
      type: dict
    bgp:
      description:
      - BGP information specific to this router.
      returned: success
      type: complex
      contains:
        asn:
          description:
          - Local BGP Autonomous System Number (ASN). Must be an RFC6996 private ASN,
            either 16-bit or 32-bit. The value will be fixed for this router resource.
            All VPN tunnels that link to this router will have the same local ASN.
          returned: success
          type: int
        advertiseMode:
          description:
          - User-specified flag to indicate which mode to use for advertisement.
          - 'Valid values of this enum field are: DEFAULT, CUSTOM .'
          returned: success
          type: str
        advertisedGroups:
          description:
          - User-specified list of prefix groups to advertise in custom mode.
          - This field can only be populated if advertiseMode is CUSTOM and is advertised
            to all peers of the router. These groups will be advertised in addition
            to any specified prefixes. Leave this field blank to advertise no custom
            groups.
          - 'This enum field has the one valid value: ALL_SUBNETS .'
          returned: success
          type: list
        advertisedIpRanges:
          description:
          - User-specified list of individual IP ranges to advertise in custom mode.
            This field can only be populated if advertiseMode is CUSTOM and is advertised
            to all peers of the router. These IP ranges will be advertised in addition
            to any specified groups.
          - Leave this field blank to advertise no custom IP ranges.
          returned: success
          type: complex
          contains:
            range:
              description:
              - The IP range to advertise. The value must be a CIDR-formatted string.
              returned: success
              type: str
            description:
              description:
              - User-specified description for the IP range.
              returned: success
              type: str
    region:
      description:
      - Region where the router resides.
      returned: success
      type: str
'''

################################################################################
# Imports
################################################################################
from ansible.module_utils.gcp_utils import navigate_hash, GcpSession, GcpModule, GcpRequest
import json

################################################################################
# Main
################################################################################


def main():
    module = GcpModule(argument_spec=dict(filters=dict(type='list', elements='str'), region=dict(required=True, type='str')))

    if not module.params['scopes']:
        module.params['scopes'] = ['https://www.googleapis.com/auth/compute']

    items = fetch_list(module, collection(module), query_options(module.params['filters']))
    if items.get('items'):
        items = items.get('items')
    else:
        items = []
    return_value = {'items': items}
    module.exit_json(**return_value)


def collection(module):
    return "https://www.googleapis.com/compute/v1/projects/{project}/regions/{region}/routers".format(**module.params)


def fetch_list(module, link, query):
    auth = GcpSession(module, 'compute')
    response = auth.get(link, params={'filter': query})
    return return_if_object(module, response)


def query_options(filters):
    if not filters:
        return ''

    if len(filters) == 1:
        return filters[0]
    else:
        queries = []
        for f in filters:
            # For multiple queries, all queries should have ()
            if f[0] != '(' and f[-1] != ')':
                queries.append("(%s)" % ''.join(f))
            else:
                queries.append(f)

        return ' '.join(queries)


def return_if_object(module, response):
    # If not found, return nothing.
    if response.status_code == 404:
        return None

    # If no content, return nothing.
    if response.status_code == 204:
        return None

    try:
        module.raise_for_status(response)
        result = response.json()
    except getattr(json.decoder, 'JSONDecodeError', ValueError) as inst:
        module.fail_json(msg="Invalid JSON response with error: %s" % inst)

    if navigate_hash(result, ['error', 'errors']):
        module.fail_json(msg=navigate_hash(result, ['error', 'errors']))

    return result


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# (c) 2019, Jordan Borean <jborean@redhat.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import os
import platform
import pytest
import sys

from units.compat.mock import MagicMock

from ansible.errors import AnsibleLookupError
from ansible.plugins.loader import lookup_loader


class FakeLDAPError(Exception):
    pass


class FakeLDAPAuthUnknownError(Exception):
    pass


class FakeLdap(object):
    SASL_AVAIL = 1
    TLS_AVAIL = 1

    SCOPE_SUBTREE = 2

    OPT_PROTOCOL_VERSION = 17
    OPT_REFERRALS = 8

    OPT_X_TLS_NEVER = 0
    OPT_X_TLS_DEMAND = 2
    OPT_X_TLS_ALLOW = 3
    OPT_X_TLS_TRY = 4

    OPT_X_TLS_CACERTFILE = 24578
    OPT_X_TLS_REQUIRE_CERT = 24582

    LDAPError = FakeLDAPError
    AUTH_UNKNOWN = FakeLDAPAuthUnknownError

    @staticmethod
    def initialize(uri, bytes_mode=None, **kwargs):
        return MagicMock()

    @staticmethod
    def set_option(option, invalue):
        pass


class FakeLdapUrl(object):

    def __init__(self, ldapUrl=None, urlscheme='ldap', hostport='', **kwargs):
        url = ldapUrl if ldapUrl else "%s://%s" % (urlscheme, hostport)
        self.urlscheme = url.split('://', 2)[0].lower()
        self._url = url

    def initializeUrl(self):
        return self._url


def fake_is_ldap_url(s):
    s_lower = s.lower()
    return s_lower.startswith("ldap://") or s_lower.startswith("ldaps://") or s_lower.startswith("ldapi://")


@pytest.fixture(autouse=True)
def laps_password():
    """Imports and the laps_password lookup with a mocks laps module for testing"""

    # Build the fake ldap and ldapurl Python modules
    fake_ldap_obj = FakeLdap()
    fake_ldap_url_obj = MagicMock()
    fake_ldap_url_obj.isLDAPUrl.side_effect = fake_is_ldap_url
    fake_ldap_url_obj.LDAPUrl.side_effect = FakeLdapUrl

    # Take a snapshot of sys.modules before we manipulate it
    orig_modules = sys.modules.copy()
    try:
        sys.modules["ldap"] = fake_ldap_obj
        sys.modules["ldapurl"] = fake_ldap_url_obj

        from ansible.plugins.lookup import laps_password

        # ensure the changes to these globals aren't persisted after each test
        orig_has_ldap = laps_password.HAS_LDAP
        orig_ldap_imp_err = laps_password.LDAP_IMP_ERR

        yield laps_password

        laps_password.HAS_LDAP = orig_has_ldap
        laps_password.LDAP_IMP_ERR = orig_ldap_imp_err
    finally:
        # Restore sys.modules back to our pre-shenanigans
        sys.modules = orig_modules


def test_missing_ldap(laps_password):
    laps_password.HAS_LDAP = False
    laps_password.LDAP_IMP_ERR = "no import for you!"

    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run(["host"], domain="test")

    assert str(err.value) == "Failed to import the required Python library (python-ldap) on %s's Python %s. See " \
                             "https://pypi.org/project/python-ldap/ for more info. Please read module documentation " \
                             "and install in the appropriate location. " \
                             "Import Error: no import for you!" % (platform.node(), sys.executable)


def test_invalid_cert_mapping():
    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run(["host"], domain="test", validate_certs="incorrect")

    assert str(err.value) == "Invalid validate_certs value 'incorrect': valid values are 'allow', 'demand', " \
                             "'never', 'try'"


def test_invalid_auth():
    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run(["host"], domain="test", auth="fail")

    assert str(err.value) == "Invalid auth value 'fail': expecting either 'gssapi', or 'simple'"


def test_gssapi_without_sasl(monkeypatch, ):
    monkeypatch.setattr("ldap.SASL_AVAIL", 0)

    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run(["host"], domain="test")

    assert str(err.value) == "Cannot use auth=gssapi when SASL is not configured with the local LDAP install"


def test_simple_auth_without_credentials():
    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run(["host"], domain="test", auth="simple")

    assert str(err.value) == "The username and password values are required when auth=simple"


def test_gssapi_auth_with_credentials():
    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run(["host"], domain="test", auth="gssapi", username="u", password="p")

    assert str(err.value) == "Explicit credentials are not supported when auth='gssapi'. Call kinit outside of Ansible"


def test_not_encrypted_without_override():
    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run(["host"], domain="dc01", auth="simple", username="test", password="test")

    assert str(err.value) == "Current configuration will result in plaintext traffic exposing credentials. Set " \
                             "auth=gssapi, scheme=ldaps, start_tls=True, or allow_plaintext=True to continue"


def test_ldaps_without_tls(monkeypatch, ):
    monkeypatch.setattr("ldap.TLS_AVAIL", 0)

    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run(["host"], domain="dc01", scheme="ldaps")

    assert str(err.value) == "Cannot use TLS as the local LDAP installed has not been configured to support it"


def test_start_tls_without_tls(monkeypatch, ):
    monkeypatch.setattr("ldap.TLS_AVAIL", 0)

    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run(["host"], domain="dc01", start_tls=True)

    assert str(err.value) == "Cannot use TLS as the local LDAP installed has not been configured to support it"


def test_normal_run(monkeypatch, laps_password):
    def get_laps_password(conn, cn, search_base):
        return "CN=%s,%s" % (cn, search_base)

    mock_ldap = MagicMock()
    mock_ldap.return_value.read_rootdse_s.return_value = {"defaultNamingContext": ["DC=domain,DC=com"]}
    monkeypatch.setattr("ldap.initialize", mock_ldap)

    mock_get_laps_password = MagicMock(side_effect=get_laps_password)
    monkeypatch.setattr(laps_password, "get_laps_password", mock_get_laps_password)

    actual = lookup_loader.get('laps_password').run(["host1", "host2"], domain="dc01")
    assert actual == ["CN=host1,DC=domain,DC=com", "CN=host2,DC=domain,DC=com"]

    # Verify the call count to get_laps_password
    assert mock_get_laps_password.call_count == 2

    # Verify the initialize() method call
    assert mock_ldap.call_count == 1
    assert mock_ldap.call_args[0] == ("ldap://dc01:389",)
    assert mock_ldap.call_args[1] == {"bytes_mode": False}

    # Verify the number of calls made to the mocked LDAP object
    assert mock_ldap.mock_calls[1][0] == "().set_option"
    assert mock_ldap.mock_calls[1][1] == (FakeLdap.OPT_PROTOCOL_VERSION, 3)

    assert mock_ldap.mock_calls[2][0] == "().set_option"
    assert mock_ldap.mock_calls[2][1] == (FakeLdap.OPT_REFERRALS, 0)

    assert mock_ldap.mock_calls[3][0] == '().sasl_gssapi_bind_s'
    assert mock_ldap.mock_calls[3][1] == ()

    assert mock_ldap.mock_calls[4][0] == "().read_rootdse_s"
    assert mock_ldap.mock_calls[4][1] == ()

    assert mock_ldap.mock_calls[5][0] == "().unbind_s"
    assert mock_ldap.mock_calls[5][1] == ()


def test_run_with_simple_auth_and_search_base(monkeypatch, laps_password):
    def get_laps_password(conn, cn, search_base):
        return "CN=%s,%s" % (cn, search_base)

    mock_ldap = MagicMock()
    monkeypatch.setattr("ldap.initialize", mock_ldap)

    mock_get_laps_password = MagicMock(side_effect=get_laps_password)
    monkeypatch.setattr(laps_password, "get_laps_password", mock_get_laps_password)

    actual = lookup_loader.get('laps_password').run(["host1", "host2"], domain="dc01", auth="simple", username="user",
                                                    password="pass", allow_plaintext=True,
                                                    search_base="OU=Workstations,DC=domain,DC=com")
    assert actual == ["CN=host1,OU=Workstations,DC=domain,DC=com", "CN=host2,OU=Workstations,DC=domain,DC=com"]

    # Verify the call count to get_laps_password
    assert mock_get_laps_password.call_count == 2

    # Verify the initialize() method call
    assert mock_ldap.call_count == 1
    assert mock_ldap.call_args[0] == ("ldap://dc01:389",)
    assert mock_ldap.call_args[1] == {"bytes_mode": False}

    # Verify the number of calls made to the mocked LDAP object
    assert mock_ldap.mock_calls[1][0] == "().set_option"
    assert mock_ldap.mock_calls[1][1] == (FakeLdap.OPT_PROTOCOL_VERSION, 3)

    assert mock_ldap.mock_calls[2][0] == "().set_option"
    assert mock_ldap.mock_calls[2][1] == (FakeLdap.OPT_REFERRALS, 0)

    assert mock_ldap.mock_calls[3][0] == '().bind_s'
    assert mock_ldap.mock_calls[3][1] == (u"user", u"pass")

    assert mock_ldap.mock_calls[4][0] == "().unbind_s"
    assert mock_ldap.mock_calls[4][1] == ()


@pytest.mark.parametrize("kwargs, expected", [
    [{"domain": "dc01"}, "ldap://dc01:389"],
    [{"domain": "dc02", "port": 1234}, "ldap://dc02:1234"],
    [{"domain": "dc03", "scheme": "ldaps"}, "ldaps://dc03:636"],
    # Verifies that an explicit URI ignores port and scheme
    [{"domain": "ldap://dc04", "port": 1234, "scheme": "ldaps"}, "ldap://dc04"],
])
def test_uri_options(monkeypatch, kwargs, expected):
    mock_ldap = MagicMock()
    monkeypatch.setattr("ldap.initialize", mock_ldap)

    lookup_loader.get('laps_password').run([], **kwargs)

    assert mock_ldap.call_count == 1
    assert mock_ldap.call_args[0] == (expected,)
    assert mock_ldap.call_args[1] == {"bytes_mode": False}


@pytest.mark.parametrize("validate, expected", [
    ["never", FakeLdap.OPT_X_TLS_NEVER],
    ["allow", FakeLdap.OPT_X_TLS_ALLOW],
    ["try", FakeLdap.OPT_X_TLS_TRY],
    ["demand", FakeLdap.OPT_X_TLS_DEMAND],
])
def test_certificate_validation(monkeypatch, validate, expected):
    mock_ldap_option = MagicMock()
    monkeypatch.setattr(FakeLdap, "set_option", mock_ldap_option)

    mock_ldap = MagicMock()
    monkeypatch.setattr("ldap.initialize", mock_ldap)

    lookup_loader.get('laps_password').run([], domain="dc01", start_tls=True, validate_certs=validate)

    assert mock_ldap_option.mock_calls[0][1] == (FakeLdap.OPT_X_TLS_REQUIRE_CERT, expected)

    assert mock_ldap.mock_calls[3][0] == "().start_tls_s"
    assert mock_ldap.mock_calls[3][1] == ()

    assert mock_ldap.mock_calls[4][0] == "().sasl_gssapi_bind_s"
    assert mock_ldap.mock_calls[4][1] == ()


def test_certificate_validate_with_custom_cacert(monkeypatch):
    mock_ldap_option = MagicMock()
    monkeypatch.setattr(FakeLdap, "set_option", mock_ldap_option)

    mock_ldap = MagicMock()
    monkeypatch.setattr("ldap.initialize", mock_ldap)
    monkeypatch.setattr(os.path, 'exists', lambda x: True)

    lookup_loader.get('laps_password').run([], domain="dc01", scheme="ldaps", cacert_file="cacert.pem")

    assert mock_ldap_option.mock_calls[0][1] == (FakeLdap.OPT_X_TLS_REQUIRE_CERT, FakeLdap.OPT_X_TLS_DEMAND)
    assert mock_ldap_option.mock_calls[1][1] == (FakeLdap.OPT_X_TLS_CACERTFILE, u"cacert.pem")

    assert mock_ldap.mock_calls[3][0] == "().sasl_gssapi_bind_s"
    assert mock_ldap.mock_calls[3][1] == ()


def test_certificate_validate_with_custom_cacert_fail(monkeypatch):
    def set_option(self, key, value):
        if key == FakeLdap.OPT_X_TLS_CACERTFILE:
            raise ValueError("set_option() failed")

    monkeypatch.setattr(FakeLdap, "set_option", set_option)
    monkeypatch.setattr(os.path, 'exists', lambda x: True)

    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run([], domain="dc01", scheme="ldaps", cacert_file="cacert.pem")

    assert str(err.value) == "Failed to set path to cacert file, this is a known issue with older OpenLDAP " \
                             "libraries on the host. Update OpenLDAP and reinstall python-ldap to continue"


@pytest.mark.parametrize("path", [
    "cacert.pem",
    "~/.certs/cacert.pem",
    "~/.certs/$USER/cacert.pem",
])
def test_certificate_invalid_path(monkeypatch, path):
    monkeypatch.setattr(os.path, 'exists', lambda x: False)
    expected_path = os.path.expanduser(os.path.expandvars(path))

    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run([], domain="dc01", scheme="ldaps", cacert_file=path)

    assert str(err.value) == "The cacert_file specified '%s' does not exist" % expected_path


def test_simple_auth_with_ldaps(monkeypatch):
    mock_ldap_option = MagicMock()
    monkeypatch.setattr(FakeLdap, "set_option", mock_ldap_option)

    mock_ldap = MagicMock()
    monkeypatch.setattr("ldap.initialize", mock_ldap)

    lookup_loader.get('laps_password').run([], domain="dc01", scheme="ldaps", auth="simple", username="user",
                                           password="pass")

    assert mock_ldap_option.mock_calls[0][1] == (FakeLdap.OPT_X_TLS_REQUIRE_CERT, FakeLdap.OPT_X_TLS_DEMAND)

    assert mock_ldap.mock_calls[3][0] == '().bind_s'
    assert mock_ldap.mock_calls[3][1] == (u"user", u"pass")

    assert mock_ldap.mock_calls[4][0] == "().read_rootdse_s"
    assert mock_ldap.mock_calls[4][1] == ()


def test_simple_auth_with_start_tls(monkeypatch):
    mock_ldap_option = MagicMock()
    monkeypatch.setattr(FakeLdap, "set_option", mock_ldap_option)

    mock_ldap = MagicMock()
    monkeypatch.setattr("ldap.initialize", mock_ldap)

    lookup_loader.get('laps_password').run([], domain="dc01", start_tls=True, auth="simple", username="user",
                                           password="pass")

    assert mock_ldap_option.mock_calls[0][1] == (FakeLdap.OPT_X_TLS_REQUIRE_CERT, FakeLdap.OPT_X_TLS_DEMAND)

    assert mock_ldap.mock_calls[3][0] == "().start_tls_s"
    assert mock_ldap.mock_calls[3][1] == ()

    assert mock_ldap.mock_calls[4][0] == '().bind_s'
    assert mock_ldap.mock_calls[4][1] == (u"user", u"pass")

    assert mock_ldap.mock_calls[5][0] == "().read_rootdse_s"
    assert mock_ldap.mock_calls[5][1] == ()


def test_start_tls_ldap_error(monkeypatch):
    mock_ldap = MagicMock()
    mock_ldap.return_value.start_tls_s.side_effect = FakeLDAPError("fake error")
    monkeypatch.setattr("ldap.initialize", mock_ldap)

    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run([], domain="dc01", start_tls=True)

    assert str(err.value) == "Failed to send StartTLS to LDAP host 'ldap://dc01:389': fake error"


def test_simple_bind_ldap_error(monkeypatch):
    mock_ldap = MagicMock()
    mock_ldap.return_value.bind_s.side_effect = FakeLDAPError("fake error")
    monkeypatch.setattr("ldap.initialize", mock_ldap)

    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run([], domain="dc01", auth="simple", username="user", password="pass",
                                               allow_plaintext=True)

    assert str(err.value) == "Failed to simple bind against LDAP host 'ldap://dc01:389': fake error"


def test_sasl_bind_ldap_error(monkeypatch):
    mock_ldap = MagicMock()
    mock_ldap.return_value.sasl_gssapi_bind_s.side_effect = FakeLDAPError("fake error")
    monkeypatch.setattr("ldap.initialize", mock_ldap)

    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run([], domain="dc01")

    assert str(err.value) == "Failed to do a sasl bind against LDAP host 'ldap://dc01:389': fake error"


def test_sasl_bind_ldap_no_mechs_error(monkeypatch):
    mock_ldap = MagicMock()
    mock_ldap.return_value.sasl_gssapi_bind_s.side_effect = FakeLDAPAuthUnknownError("no mechs")
    monkeypatch.setattr("ldap.initialize", mock_ldap)

    with pytest.raises(AnsibleLookupError) as err:
        lookup_loader.get('laps_password').run([], domain="dc01")

    assert str(err.value) == "Failed to do a sasl bind against LDAP host 'ldap://dc01:389', the GSSAPI mech is " \
                             "not installed: no mechs"


def test_get_password_valid(laps_password):
    mock_conn = MagicMock()
    mock_conn.search_s.return_value = [
        ("CN=server,DC=domain,DC=local",
         {"ms-Mcs-AdmPwd": ["pass"], "distinguishedName": ["CN=server,DC=domain,DC=local"]}),
        # Replicates the 3 extra entries AD returns that aren't server objects
        (None, ["ldap://ForestDnsZones.domain.com/DC=ForestDnsZones,DC=domain,DC=com"]),
        (None, ["ldap://DomainDnsZones.domain.com/DC=DomainDnsZones,DC=domain,DC=com"]),
        (None, ["ldap://domain.com/CN=Configuration,DC=domain,DC=com"]),
    ]

    actual = laps_password.get_laps_password(mock_conn, "server", "DC=domain,DC=local")
    assert actual == "pass"

    assert len(mock_conn.method_calls) == 1
    assert mock_conn.method_calls[0][0] == "search_s"
    assert mock_conn.method_calls[0][1] == ("DC=domain,DC=local", FakeLdap.SCOPE_SUBTREE,
                                            "(&(objectClass=computer)(CN=server))")
    assert mock_conn.method_calls[0][2] == {"attrlist": ["distinguishedName", "ms-Mcs-AdmPwd"]}


def test_get_password_laps_not_configured(laps_password):
    mock_conn = MagicMock()
    mock_conn.search_s.return_value = [
        ("CN=server,DC=domain,DC=local", {"distinguishedName": ["CN=server,DC=domain,DC=local"]}),
        (None, ["ldap://ForestDnsZones.domain.com/DC=ForestDnsZones,DC=domain,DC=com"]),
        (None, ["ldap://DomainDnsZones.domain.com/DC=DomainDnsZones,DC=domain,DC=com"]),
        (None, ["ldap://domain.com/CN=Configuration,DC=domain,DC=com"]),
    ]

    with pytest.raises(AnsibleLookupError) as err:
        laps_password.get_laps_password(mock_conn, "server2", "DC=test,DC=local")
    assert str(err.value) == \
        "The server 'CN=server,DC=domain,DC=local' did not have the LAPS attribute 'ms-Mcs-AdmPwd'"

    assert len(mock_conn.method_calls) == 1
    assert mock_conn.method_calls[0][0] == "search_s"
    assert mock_conn.method_calls[0][1] == ("DC=test,DC=local", FakeLdap.SCOPE_SUBTREE,
                                            "(&(objectClass=computer)(CN=server2))")
    assert mock_conn.method_calls[0][2] == {"attrlist": ["distinguishedName", "ms-Mcs-AdmPwd"]}


def test_get_password_no_results(laps_password):
    mock_conn = MagicMock()
    mock_conn.search_s.return_value = [
        (None, ["ldap://ForestDnsZones.domain.com/DC=ForestDnsZones,DC=domain,DC=com"]),
        (None, ["ldap://DomainDnsZones.domain.com/DC=DomainDnsZones,DC=domain,DC=com"]),
        (None, ["ldap://domain.com/CN=Configuration,DC=domain,DC=com"]),
    ]

    with pytest.raises(AnsibleLookupError) as err:
        laps_password.get_laps_password(mock_conn, "server", "DC=domain,DC=local")
    assert str(err.value) == "Failed to find the server 'server' in the base 'DC=domain,DC=local'"

    assert len(mock_conn.method_calls) == 1
    assert mock_conn.method_calls[0][0] == "search_s"
    assert mock_conn.method_calls[0][1] == ("DC=domain,DC=local", FakeLdap.SCOPE_SUBTREE,
                                            "(&(objectClass=computer)(CN=server))")
    assert mock_conn.method_calls[0][2] == {"attrlist": ["distinguishedName", "ms-Mcs-AdmPwd"]}


def test_get_password_multiple_results(laps_password):
    mock_conn = MagicMock()
    mock_conn.search_s.return_value = [
        ("CN=server,OU=Workstations,DC=domain,DC=local",
         {"ms-Mcs-AdmPwd": ["pass"], "distinguishedName": ["CN=server,OU=Workstations,DC=domain,DC=local"]}),
        ("CN=server,OU=Servers,DC=domain,DC=local",
         {"ms-Mcs-AdmPwd": ["pass"], "distinguishedName": ["CN=server,OU=Servers,DC=domain,DC=local"]}),
        (None, ["ldap://ForestDnsZones.domain.com/DC=ForestDnsZones,DC=domain,DC=com"]),
        (None, ["ldap://DomainDnsZones.domain.com/DC=DomainDnsZones,DC=domain,DC=com"]),
        (None, ["ldap://domain.com/CN=Configuration,DC=domain,DC=com"]),
    ]

    with pytest.raises(AnsibleLookupError) as err:
        laps_password.get_laps_password(mock_conn, "server", "DC=domain,DC=local")
    assert str(err.value) == \
        "Found too many results for the server 'server' in the base 'DC=domain,DC=local'. Specify a more explicit " \
        "search base for the server required. Found servers 'CN=server,OU=Workstations,DC=domain,DC=local', " \
        "'CN=server,OU=Servers,DC=domain,DC=local'"

    assert len(mock_conn.method_calls) == 1
    assert mock_conn.method_calls[0][0] == "search_s"
    assert mock_conn.method_calls[0][1] == ("DC=domain,DC=local", FakeLdap.SCOPE_SUBTREE,
                                            "(&(objectClass=computer)(CN=server))")
    assert mock_conn.method_calls[0][2] == {"attrlist": ["distinguishedName", "ms-Mcs-AdmPwd"]}



# (c) 2016, Adrian Likins <alikins@redhat.com>
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.

# Make coding more python3-ish
from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

import ansible
import os
import shutil
import tarfile
import tempfile
import yaml

from ansible import context
from ansible.cli.arguments import optparse_helpers as opt_help
from ansible.cli.galaxy import GalaxyCLI
from ansible.errors import AnsibleError, AnsibleOptionsError
from ansible.module_utils.six import PY3
from ansible.utils import context_objects as co
from units.compat import unittest
from units.compat.mock import call, patch


class TestGalaxy(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        '''creating prerequisites for installing a role; setUpClass occurs ONCE whereas setUp occurs with every method tested.'''
        # class data for easy viewing: role_dir, role_tar, role_name, role_req, role_path

        cls.temp_dir = tempfile.mkdtemp(prefix='ansible-test_galaxy-')
        os.chdir(cls.temp_dir)

        if os.path.exists("./delete_me"):
            shutil.rmtree("./delete_me")

        # creating framework for a role
        gc = GalaxyCLI(args=["ansible-galaxy", "init", "--offline", "delete_me"])
        gc.run()
        cls.role_dir = "./delete_me"
        cls.role_name = "delete_me"

        # making a temp dir for role installation
        cls.role_path = os.path.join(tempfile.mkdtemp(), "roles")
        if not os.path.isdir(cls.role_path):
            os.makedirs(cls.role_path)

        # creating a tar file name for class data
        cls.role_tar = './delete_me.tar.gz'
        cls.makeTar(cls.role_tar, cls.role_dir)

        # creating a temp file with installation requirements
        cls.role_req = './delete_me_requirements.yml'
        fd = open(cls.role_req, "w")
        fd.write("- 'src': '%s'\n  'name': '%s'\n  'path': '%s'" % (cls.role_tar, cls.role_name, cls.role_path))
        fd.close()

    @classmethod
    def makeTar(cls, output_file, source_dir):
        ''' used for making a tarfile from a role directory '''
        # adding directory into a tar file
        try:
            tar = tarfile.open(output_file, "w:gz")
            tar.add(source_dir, arcname=os.path.basename(source_dir))
        except AttributeError:  # tarfile obj. has no attribute __exit__ prior to python 2.    7
            pass
        finally:  # ensuring closure of tarfile obj
            tar.close()

    @classmethod
    def tearDownClass(cls):
        '''After tests are finished removes things created in setUpClass'''
        # deleting the temp role directory
        if os.path.exists(cls.role_dir):
            shutil.rmtree(cls.role_dir)
        if os.path.exists(cls.role_req):
            os.remove(cls.role_req)
        if os.path.exists(cls.role_tar):
            os.remove(cls.role_tar)
        if os.path.isdir(cls.role_path):
            shutil.rmtree(cls.role_path)

        os.chdir('/')
        shutil.rmtree(cls.temp_dir)

    def setUp(self):
        # Reset the stored command line args
        co.GlobalCLIArgs._Singleton__instance = None
        self.default_args = ['ansible-galaxy']

    def tearDown(self):
        # Reset the stored command line args
        co.GlobalCLIArgs._Singleton__instance = None

    def test_init(self):
        galaxy_cli = GalaxyCLI(args=self.default_args)
        self.assertTrue(isinstance(galaxy_cli, GalaxyCLI))

    def test_display_min(self):
        gc = GalaxyCLI(args=self.default_args)
        role_info = {'name': 'some_role_name'}
        display_result = gc._display_role_info(role_info)
        self.assertTrue(display_result.find('some_role_name') > -1)

    def test_display_galaxy_info(self):
        gc = GalaxyCLI(args=self.default_args)
        galaxy_info = {}
        role_info = {'name': 'some_role_name',
                     'galaxy_info': galaxy_info}
        display_result = gc._display_role_info(role_info)
        if display_result.find('\n\tgalaxy_info:') == -1:
            self.fail('Expected galaxy_info to be indented once')

    def test_run(self):
        ''' verifies that the GalaxyCLI object's api is created and that execute() is called. '''
        gc = GalaxyCLI(args=["ansible-galaxy", "install", "--ignore-errors", "imaginary_role"])
        with patch.object(ansible.cli.CLI, "execute", return_value=None) as mock_ex:
            with patch.object(ansible.cli.CLI, "run", return_value=None) as mock_run:
                gc.run()
                # testing
                self.assertIsInstance(gc.galaxy, ansible.galaxy.Galaxy)
                self.assertEqual(mock_run.call_count, 1)
                self.assertTrue(isinstance(gc.api, ansible.galaxy.api.GalaxyAPI))
                self.assertEqual(mock_ex.call_count, 1)

    def test_execute_remove(self):
        # installing role
        gc = GalaxyCLI(args=["ansible-galaxy", "install", "-p", self.role_path, "-r", self.role_req, '--force'])
        gc.run()

        # location where the role was installed
        role_file = os.path.join(self.role_path, self.role_name)

        # removing role
        # Have to reset the arguments in the context object manually since we're doing the
        # equivalent of running the command line program twice
        co.GlobalCLIArgs._Singleton__instance = None
        gc = GalaxyCLI(args=["ansible-galaxy", "remove", role_file, self.role_name])
        gc.run()

        # testing role was removed
        removed_role = not os.path.exists(role_file)
        self.assertTrue(removed_role)

    def test_exit_without_ignore_without_flag(self):
        ''' tests that GalaxyCLI exits with the error specified if the --ignore-errors flag is not used '''
        gc = GalaxyCLI(args=["ansible-galaxy", "install", "--server=None", "fake_role_name"])
        with patch.object(ansible.utils.display.Display, "display", return_value=None) as mocked_display:
            # testing that error expected is raised
            self.assertRaises(AnsibleError, gc.run)
            self.assertTrue(mocked_display.called_once_with("- downloading role 'fake_role_name', owned by "))

    def test_exit_without_ignore_with_flag(self):
        ''' tests that GalaxyCLI exits without the error specified if the --ignore-errors flag is used  '''
        # testing with --ignore-errors flag
        gc = GalaxyCLI(args=["ansible-galaxy", "install", "--server=None", "fake_role_name", "--ignore-errors"])
        with patch.object(ansible.utils.display.Display, "display", return_value=None) as mocked_display:
            gc.run()
            self.assertTrue(mocked_display.called_once_with("- downloading role 'fake_role_name', owned by "))

    def run_parse_common(self, galaxycli_obj, action):
        with patch.object(opt_help.SortedOptParser, "set_usage") as mocked_usage:
            galaxycli_obj.parse()

            # checking that the common results of parse() for all possible actions have been created/called
            self.assertIsInstance(galaxycli_obj.parser, opt_help.SortedOptParser)
            formatted_call = {
                'import': 'usage: %prog import [options] github_user github_repo',
                'delete': 'usage: %prog delete [options] github_user github_repo',
                'info': 'usage: %prog info [options] role_name[,version]',
                'init': 'usage: %prog init [options] role_name',
                'install': 'usage: %prog install [options] [-r FILE | role_name(s)[,version] | scm+role_repo_url[,version] | tar_file(s)]',
                'list': 'usage: %prog list [role_name]',
                'login': 'usage: %prog login [options]',
                'remove': 'usage: %prog remove role1 role2 ...',
                'search': ('usage: %prog search [searchterm1 searchterm2] [--galaxy-tags galaxy_tag1,galaxy_tag2] [--platforms platform1,platform2] '
                           '[--author username]'),
                'setup': 'usage: %prog setup [options] source github_user github_repo secret',
            }

            first_call = 'usage: %prog [delete|import|info|init|install|list|login|remove|search|setup] [--help] [options] ...'
            second_call = formatted_call[action]
            calls = [call(first_call), call(second_call)]
            mocked_usage.assert_has_calls(calls)

    def test_parse_no_action(self):
        ''' testing the options parser when no action is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", ""])
        self.assertRaises(AnsibleOptionsError, gc.parse)

    def test_parse_invalid_action(self):
        ''' testing the options parser when an invalid action is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", "NOT_ACTION"])
        self.assertRaises(AnsibleOptionsError, gc.parse)

    def test_parse_delete(self):
        ''' testing the options parser when the action 'delete' is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", "delete"])
        self.run_parse_common(gc, "delete")
        self.assertEqual(context.CLIARGS['verbosity'], 0)

    def test_parse_import(self):
        ''' testing the options parser when the action 'import' is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", "import"])
        self.run_parse_common(gc, "import")
        self.assertEqual(context.CLIARGS['wait'], True)
        self.assertEqual(context.CLIARGS['reference'], None)
        self.assertEqual(context.CLIARGS['check_status'], False)
        self.assertEqual(context.CLIARGS['verbosity'], 0)

    def test_parse_info(self):
        ''' testing the options parser when the action 'info' is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", "info"])
        self.run_parse_common(gc, "info")
        self.assertEqual(context.CLIARGS['offline'], False)

    def test_parse_init(self):
        ''' testing the options parser when the action 'init' is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", "init"])
        self.run_parse_common(gc, "init")
        self.assertEqual(context.CLIARGS['offline'], False)
        self.assertEqual(context.CLIARGS['force'], False)

    def test_parse_install(self):
        ''' testing the options parser when the action 'install' is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", "install"])
        self.run_parse_common(gc, "install")
        self.assertEqual(context.CLIARGS['ignore_errors'], False)
        self.assertEqual(context.CLIARGS['no_deps'], False)
        self.assertEqual(context.CLIARGS['role_file'], None)
        self.assertEqual(context.CLIARGS['force'], False)

    def test_parse_list(self):
        ''' testing the options parser when the action 'list' is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", "list"])
        self.run_parse_common(gc, "list")
        self.assertEqual(context.CLIARGS['verbosity'], 0)

    def test_parse_login(self):
        ''' testing the options parser when the action 'login' is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", "login"])
        self.run_parse_common(gc, "login")
        self.assertEqual(context.CLIARGS['verbosity'], 0)
        self.assertEqual(context.CLIARGS['token'], None)

    def test_parse_remove(self):
        ''' testing the options parser when the action 'remove' is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", "remove"])
        self.run_parse_common(gc, "remove")
        self.assertEqual(context.CLIARGS['verbosity'], 0)

    def test_parse_search(self):
        ''' testing the options parswer when the action 'search' is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", "search"])
        self.run_parse_common(gc, "search")
        self.assertEqual(context.CLIARGS['platforms'], None)
        self.assertEqual(context.CLIARGS['galaxy_tags'], None)
        self.assertEqual(context.CLIARGS['author'], None)

    def test_parse_setup(self):
        ''' testing the options parser when the action 'setup' is given '''
        gc = GalaxyCLI(args=["ansible-galaxy", "setup"])
        self.run_parse_common(gc, "setup")

        self.assertEqual(context.CLIARGS['verbosity'], 0)
        self.assertEqual(context.CLIARGS['remove_id'], None)
        self.assertEqual(context.CLIARGS['setup_list'], False)


class ValidRoleTests(object):

    expected_role_dirs = ('defaults', 'files', 'handlers', 'meta', 'tasks', 'templates', 'vars', 'tests')

    @classmethod
    def setUpRole(cls, role_name, galaxy_args=None, skeleton_path=None):
        if galaxy_args is None:
            galaxy_args = []

        if skeleton_path is not None:
            cls.role_skeleton_path = skeleton_path
            galaxy_args += ['--role-skeleton', skeleton_path]

        # Make temp directory for testing
        cls.test_dir = tempfile.mkdtemp()
        if not os.path.isdir(cls.test_dir):
            os.makedirs(cls.test_dir)

        cls.role_dir = os.path.join(cls.test_dir, role_name)
        cls.role_name = role_name

        # create role using default skeleton
        gc = GalaxyCLI(args=['ansible-galaxy', 'init', '-c', '--offline'] + galaxy_args + ['--init-path', cls.test_dir, cls.role_name])
        gc.run()
        cls.gc = gc

        if skeleton_path is None:
            cls.role_skeleton_path = gc.galaxy.default_role_skeleton_path

    @classmethod
    def tearDownClass(cls):
        if os.path.isdir(cls.test_dir):
            shutil.rmtree(cls.test_dir)

    def test_metadata(self):
        with open(os.path.join(self.role_dir, 'meta', 'main.yml'), 'r') as mf:
            metadata = yaml.safe_load(mf)
        self.assertIn('galaxy_info', metadata, msg='unable to find galaxy_info in metadata')
        self.assertIn('dependencies', metadata, msg='unable to find dependencies in metadata')

    def test_readme(self):
        readme_path = os.path.join(self.role_dir, 'README.md')
        self.assertTrue(os.path.exists(readme_path), msg='Readme doesn\'t exist')

    def test_main_ymls(self):
        need_main_ymls = set(self.expected_role_dirs) - set(['meta', 'tests', 'files', 'templates'])
        for d in need_main_ymls:
            main_yml = os.path.join(self.role_dir, d, 'main.yml')
            self.assertTrue(os.path.exists(main_yml))
            expected_string = "---\n# {0} file for {1}".format(d, self.role_name)
            with open(main_yml, 'r') as f:
                self.assertEqual(expected_string, f.read().strip())

    def test_role_dirs(self):
        for d in self.expected_role_dirs:
            self.assertTrue(os.path.isdir(os.path.join(self.role_dir, d)), msg="Expected role subdirectory {0} doesn't exist".format(d))

    def test_travis_yml(self):
        with open(os.path.join(self.role_dir, '.travis.yml'), 'r') as f:
            contents = f.read()

        with open(os.path.join(self.role_skeleton_path, '.travis.yml'), 'r') as f:
            expected_contents = f.read()

        self.assertEqual(expected_contents, contents, msg='.travis.yml does not match expected')

    def test_readme_contents(self):
        with open(os.path.join(self.role_dir, 'README.md'), 'r') as readme:
            contents = readme.read()

        with open(os.path.join(self.role_skeleton_path, 'README.md'), 'r') as f:
            expected_contents = f.read()

        self.assertEqual(expected_contents, contents, msg='README.md does not match expected')

    def test_test_yml(self):
        with open(os.path.join(self.role_dir, 'tests', 'test.yml'), 'r') as f:
            test_playbook = yaml.safe_load(f)
        print(test_playbook)
        self.assertEqual(len(test_playbook), 1)
        self.assertEqual(test_playbook[0]['hosts'], 'localhost')
        self.assertEqual(test_playbook[0]['remote_user'], 'root')
        self.assertListEqual(test_playbook[0]['roles'], [self.role_name], msg='The list of roles included in the test play doesn\'t match')


class TestGalaxyInitDefault(unittest.TestCase, ValidRoleTests):

    @classmethod
    def setUpClass(cls):
        cls.setUpRole(role_name='delete_me')

    def test_metadata_contents(self):
        with open(os.path.join(self.role_dir, 'meta', 'main.yml'), 'r') as mf:
            metadata = yaml.safe_load(mf)
        self.assertEqual(metadata.get('galaxy_info', dict()).get('author'), 'your name', msg='author was not set properly in metadata')


class TestGalaxyInitAPB(unittest.TestCase, ValidRoleTests):

    @classmethod
    def setUpClass(cls):
        cls.setUpRole('delete_me_apb', galaxy_args=['--type=apb'])

    def test_metadata_apb_tag(self):
        with open(os.path.join(self.role_dir, 'meta', 'main.yml'), 'r') as mf:
            metadata = yaml.safe_load(mf)
        self.assertIn('apb', metadata.get('galaxy_info', dict()).get('galaxy_tags', []), msg='apb tag not set in role metadata')

    def test_metadata_contents(self):
        with open(os.path.join(self.role_dir, 'meta', 'main.yml'), 'r') as mf:
            metadata = yaml.safe_load(mf)
        self.assertEqual(metadata.get('galaxy_info', dict()).get('author'), 'your name', msg='author was not set properly in metadata')

    def test_apb_yml(self):
        self.assertTrue(os.path.exists(os.path.join(self.role_dir, 'apb.yml')), msg='apb.yml was not created')

    def test_test_yml(self):
        with open(os.path.join(self.role_dir, 'tests', 'test.yml'), 'r') as f:
            test_playbook = yaml.safe_load(f)
        print(test_playbook)
        self.assertEqual(len(test_playbook), 1)
        self.assertEqual(test_playbook[0]['hosts'], 'localhost')
        self.assertFalse(test_playbook[0]['gather_facts'])
        self.assertEqual(test_playbook[0]['connection'], 'local')
        self.assertIsNone(test_playbook[0]['tasks'], msg='We\'re expecting an unset list of tasks in test.yml')


class TestGalaxyInitContainer(unittest.TestCase, ValidRoleTests):

    @classmethod
    def setUpClass(cls):
        cls.setUpRole('delete_me_container', galaxy_args=['--type=container'])

    def test_metadata_container_tag(self):
        with open(os.path.join(self.role_dir, 'meta', 'main.yml'), 'r') as mf:
            metadata = yaml.safe_load(mf)
        self.assertIn('container', metadata.get('galaxy_info', dict()).get('galaxy_tags', []), msg='container tag not set in role metadata')

    def test_metadata_contents(self):
        with open(os.path.join(self.role_dir, 'meta', 'main.yml'), 'r') as mf:
            metadata = yaml.safe_load(mf)
        self.assertEqual(metadata.get('galaxy_info', dict()).get('author'), 'your name', msg='author was not set properly in metadata')

    def test_meta_container_yml(self):
        self.assertTrue(os.path.exists(os.path.join(self.role_dir, 'meta', 'container.yml')), msg='container.yml was not created')

    def test_test_yml(self):
        with open(os.path.join(self.role_dir, 'tests', 'test.yml'), 'r') as f:
            test_playbook = yaml.safe_load(f)
        print(test_playbook)
        self.assertEqual(len(test_playbook), 1)
        self.assertEqual(test_playbook[0]['hosts'], 'localhost')
        self.assertFalse(test_playbook[0]['gather_facts'])
        self.assertEqual(test_playbook[0]['connection'], 'local')
        self.assertIsNone(test_playbook[0]['tasks'], msg='We\'re expecting an unset list of tasks in test.yml')


class TestGalaxyInitSkeleton(unittest.TestCase, ValidRoleTests):

    @classmethod
    def setUpClass(cls):
        role_skeleton_path = os.path.join(os.path.split(__file__)[0], 'test_data', 'role_skeleton')
        cls.setUpRole('delete_me_skeleton', skeleton_path=role_skeleton_path)

    def test_empty_files_dir(self):
        files_dir = os.path.join(self.role_dir, 'files')
        self.assertTrue(os.path.isdir(files_dir))
        self.assertListEqual(os.listdir(files_dir), [], msg='we expect the files directory to be empty, is ignore working?')

    def test_template_ignore_jinja(self):
        test_conf_j2 = os.path.join(self.role_dir, 'templates', 'test.conf.j2')
        self.assertTrue(os.path.exists(test_conf_j2), msg="The test.conf.j2 template doesn't seem to exist, is it being rendered as test.conf?")
        with open(test_conf_j2, 'r') as f:
            contents = f.read()
        expected_contents = '[defaults]\ntest_key = {{ test_variable }}'
        self.assertEqual(expected_contents, contents.strip(), msg="test.conf.j2 doesn't contain what it should, is it being rendered?")

    def test_template_ignore_jinja_subfolder(self):
        test_conf_j2 = os.path.join(self.role_dir, 'templates', 'subfolder', 'test.conf.j2')
        self.assertTrue(os.path.exists(test_conf_j2), msg="The test.conf.j2 template doesn't seem to exist, is it being rendered as test.conf?")
        with open(test_conf_j2, 'r') as f:
            contents = f.read()
        expected_contents = '[defaults]\ntest_key = {{ test_variable }}'
        self.assertEqual(expected_contents, contents.strip(), msg="test.conf.j2 doesn't contain what it should, is it being rendered?")

    def test_template_ignore_similar_folder(self):
        self.assertTrue(os.path.exists(os.path.join(self.role_dir, 'templates_extra', 'templates.txt')))

    def test_skeleton_option(self):
        self.assertEquals(self.role_skeleton_path, context.CLIARGS['role_skeleton'], msg='Skeleton path was not parsed properly from the command line')



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2018 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_system_sdn_connector
short_description: Configure connection to SDN Connector.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure system feature and sdn_connector category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    system_sdn_connector:
        description:
            - Configure connection to SDN Connector.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            access-key:
                description:
                    - AWS access key ID.
            azure-region:
                description:
                    - Azure server region.
                choices:
                    - global
                    - china
                    - germany
                    - usgov
            client-id:
                description:
                    - Azure client ID (application ID).
            client-secret:
                description:
                    - Azure client secret (application key).
            compartment-id:
                description:
                    - Compartment ID.
            external-ip:
                description:
                    - Configure GCP external IP.
                suboptions:
                    name:
                        description:
                            - External IP name.
                        required: true
            gcp-project:
                description:
                    - GCP project name.
            key-passwd:
                description:
                    - Private key password.
            name:
                description:
                    - SDN connector name.
                required: true
            nic:
                description:
                    - Configure Azure network interface.
                suboptions:
                    ip:
                        description:
                            - Configure IP configuration.
                        suboptions:
                            name:
                                description:
                                    - IP configuration name.
                                required: true
                            public-ip:
                                description:
                                    - Public IP name.
                    name:
                        description:
                            - Network interface name.
                        required: true
            oci-cert:
                description:
                    - OCI certificate. Source certificate.local.name.
            oci-fingerprint:
                description:
                    - OCI pubkey fingerprint.
            oci-region:
                description:
                    - OCI server region.
                choices:
                    - phoenix
                    - ashburn
                    - frankfurt
                    - london
            password:
                description:
                    - Password of the remote SDN connector as login credentials.
            private-key:
                description:
                    - Private key of GCP service account.
            region:
                description:
                    - AWS region name.
            resource-group:
                description:
                    - Azure resource group.
            route:
                description:
                    - Configure GCP route.
                suboptions:
                    name:
                        description:
                            - Route name.
                        required: true
            route-table:
                description:
                    - Configure Azure route table.
                suboptions:
                    name:
                        description:
                            - Route table name.
                        required: true
                    route:
                        description:
                            - Configure Azure route.
                        suboptions:
                            name:
                                description:
                                    - Route name.
                                required: true
                            next-hop:
                                description:
                                    - Next hop address.
            secret-key:
                description:
                    - AWS secret access key.
            server:
                description:
                    - Server address of the remote SDN connector.
            server-port:
                description:
                    - Port number of the remote SDN connector.
            service-account:
                description:
                    - GCP service account email.
            status:
                description:
                    - Enable/disable connection to the remote SDN connector.
                choices:
                    - disable
                    - enable
            subscription-id:
                description:
                    - Azure subscription ID.
            tenant-id:
                description:
                    - Tenant ID (directory ID).
            type:
                description:
                    - Type of SDN connector.
                choices:
                    - aci
                    - aws
                    - azure
                    - nsx
                    - nuage
                    - oci
                    - gcp
            update-interval:
                description:
                    - Dynamic object update interval (0 - 3600 sec, 0 means disabled, default = 60).
            use-metadata-iam:
                description:
                    - Enable/disable using IAM role from metadata to call API.
                choices:
                    - disable
                    - enable
            user-id:
                description:
                    - User ID.
            username:
                description:
                    - Username of the remote SDN connector as login credentials.
            vpc-id:
                description:
                    - AWS VPC ID.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Configure connection to SDN Connector.
    fortios_system_sdn_connector:
      host:  "{{  host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{  vdom }}"
      system_sdn_connector:
        state: "present"
        access-key: "<your_own_value>"
        azure-region: "global"
        client-id: "<your_own_value>"
        client-secret: "<your_own_value>"
        compartment-id: "<your_own_value>"
        external-ip:
         -
            name: "default_name_9"
        gcp-project: "<your_own_value>"
        key-passwd: "<your_own_value>"
        name: "default_name_12"
        nic:
         -
            ip:
             -
                name: "default_name_15"
                public-ip: "<your_own_value>"
            name: "default_name_17"
        oci-cert: "<your_own_value> (source certificate.local.name)"
        oci-fingerprint: "<your_own_value>"
        oci-region: "phoenix"
        password: "<your_own_value>"
        private-key: "<your_own_value>"
        region: "<your_own_value>"
        resource-group: "<your_own_value>"
        route:
         -
            name: "default_name_26"
        route-table:
         -
            name: "default_name_28"
            route:
             -
                name: "default_name_30"
                next-hop: "<your_own_value>"
        secret-key: "<your_own_value>"
        server: "192.168.100.40"
        server-port: "34"
        service-account: "<your_own_value>"
        status: "disable"
        subscription-id: "<your_own_value>"
        tenant-id: "<your_own_value>"
        type: "aci"
        update-interval: "40"
        use-metadata-iam: "disable"
        user-id: "<your_own_value>"
        username: "<your_own_value>"
        vpc-id: "<your_own_value>"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "key1"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_system_sdn_connector_data(json):
    option_list = ['access-key', 'azure-region', 'client-id',
                   'client-secret', 'compartment-id', 'external-ip',
                   'gcp-project', 'key-passwd', 'name',
                   'nic', 'oci-cert', 'oci-fingerprint',
                   'oci-region', 'password', 'private-key',
                   'region', 'resource-group', 'route',
                   'route-table', 'secret-key', 'server',
                   'server-port', 'service-account', 'status',
                   'subscription-id', 'tenant-id', 'type',
                   'update-interval', 'use-metadata-iam', 'user-id',
                   'username', 'vpc-id']
    dictionary = {}

    for attribute in option_list:
        if attribute in json:
            dictionary[attribute] = json[attribute]

    return dictionary


def system_sdn_connector(data, fos):
    vdom = data['vdom']
    system_sdn_connector_data = data['system_sdn_connector']
    filtered_data = filter_system_sdn_connector_data(system_sdn_connector_data)
    if system_sdn_connector_data['state'] == "present":
        return fos.set('system',
                       'sdn-connector',
                       data=filtered_data,
                       vdom=vdom)

    elif system_sdn_connector_data['state'] == "absent":
        return fos.delete('system',
                          'sdn-connector',
                          mkey=filtered_data['name'],
                          vdom=vdom)


def fortios_system(data, fos):
    login(data)

    methodlist = ['system_sdn_connector']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "system_sdn_connector": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "access-key": {"required": False, "type": "str"},
                "azure-region": {"required": False, "type": "str",
                                 "choices": ["global", "china", "germany",
                                             "usgov"]},
                "client-id": {"required": False, "type": "str"},
                "client-secret": {"required": False, "type": "str"},
                "compartment-id": {"required": False, "type": "str"},
                "external-ip": {"required": False, "type": "list",
                                "options": {
                                    "name": {"required": True, "type": "str"}
                                }},
                "gcp-project": {"required": False, "type": "str"},
                "key-passwd": {"required": False, "type": "str"},
                "name": {"required": True, "type": "str"},
                "nic": {"required": False, "type": "list",
                        "options": {
                            "ip": {"required": False, "type": "list",
                                   "options": {
                                       "name": {"required": True, "type": "str"},
                                       "public-ip": {"required": False, "type": "str"}
                                   }},
                            "name": {"required": True, "type": "str"}
                        }},
                "oci-cert": {"required": False, "type": "str"},
                "oci-fingerprint": {"required": False, "type": "str"},
                "oci-region": {"required": False, "type": "str",
                               "choices": ["phoenix", "ashburn", "frankfurt",
                                           "london"]},
                "password": {"required": False, "type": "str"},
                "private-key": {"required": False, "type": "str"},
                "region": {"required": False, "type": "str"},
                "resource-group": {"required": False, "type": "str"},
                "route": {"required": False, "type": "list",
                          "options": {
                              "name": {"required": True, "type": "str"}
                          }},
                "route-table": {"required": False, "type": "list",
                                "options": {
                                    "name": {"required": True, "type": "str"},
                                    "route": {"required": False, "type": "list",
                                              "options": {
                                                  "name": {"required": True, "type": "str"},
                                                  "next-hop": {"required": False, "type": "str"}
                                              }}
                                }},
                "secret-key": {"required": False, "type": "str"},
                "server": {"required": False, "type": "str"},
                "server-port": {"required": False, "type": "int"},
                "service-account": {"required": False, "type": "str"},
                "status": {"required": False, "type": "str",
                           "choices": ["disable", "enable"]},
                "subscription-id": {"required": False, "type": "str"},
                "tenant-id": {"required": False, "type": "str"},
                "type": {"required": False, "type": "str",
                         "choices": ["aci", "aws", "azure",
                                     "nsx", "nuage", "oci",
                                     "gcp"]},
                "update-interval": {"required": False, "type": "int"},
                "use-metadata-iam": {"required": False, "type": "str",
                                     "choices": ["disable", "enable"]},
                "user-id": {"required": False, "type": "str"},
                "username": {"required": False, "type": "str"},
                "vpc-id": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_system(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
# Copyright: (c) 2018, Ansible Project
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

# Make coding more python3-ish
from __future__ import (absolute_import, division)
__metaclass__ = type

import pytest

from ansible.cli.arguments import optparse_helpers as opt_help


class TestOptparseHelpersVersion:

    def test_version(self):
        ver = opt_help.version('ansible-cli-test')
        assert 'ansible-cli-test' in ver
        assert 'python version' in ver



#!/usr/bin/python
from __future__ import (absolute_import, division, print_function)
# Copyright 2019 Fortinet, Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.
#
# the lib use python logging can get it if the following is set in your
# Ansible config.

__metaclass__ = type

ANSIBLE_METADATA = {'status': ['preview'],
                    'supported_by': 'community',
                    'metadata_version': '1.1'}

DOCUMENTATION = '''
---
module: fortios_extender_controller_extender
short_description: Extender controller configuration in Fortinet's FortiOS and FortiGate.
description:
    - This module is able to configure a FortiGate or FortiOS by
      allowing the user to configure extender_controller feature and extender category.
      Examples includes all options and need to be adjusted to datasources before usage.
      Tested with FOS v6.0.2
version_added: "2.8"
author:
    - Miguel Angel Munoz (@mamunozgonzalez)
    - Nicolas Thomas (@thomnico)
notes:
    - Requires fortiosapi library developed by Fortinet
    - Run as a local_action in your playbook
requirements:
    - fortiosapi>=0.9.8
options:
    host:
       description:
            - FortiOS or FortiGate ip address.
       required: true
    username:
        description:
            - FortiOS or FortiGate username.
        required: true
    password:
        description:
            - FortiOS or FortiGate password.
        default: ""
    vdom:
        description:
            - Virtual domain, among those defined previously. A vdom is a
              virtual instance of the FortiGate that can be configured and
              used as a different unit.
        default: root
    https:
        description:
            - Indicates if the requests towards FortiGate must use HTTPS
              protocol
        type: bool
        default: false
    extender_controller_extender:
        description:
            - Extender controller configuration.
        default: null
        suboptions:
            state:
                description:
                    - Indicates whether to create or remove the object
                choices:
                    - present
                    - absent
            aaa-shared-secret:
                description:
                    - AAA shared secret.
            access-point-name:
                description:
                    - Access point name(APN).
            admin:
                description:
                    - FortiExtender Administration (enable or disable).
                choices:
                    - disable
                    - discovered
                    - enable
            at-dial-script:
                description:
                    - Initialization AT commands specific to the MODEM.
            billing-start-day:
                description:
                    - Billing start day.
            cdma-aaa-spi:
                description:
                    - CDMA AAA SPI.
            cdma-ha-spi:
                description:
                    - CDMA HA SPI.
            cdma-nai:
                description:
                    - NAI for CDMA MODEMS.
            conn-status:
                description:
                    - Connection status.
            description:
                description:
                    - Description.
            dial-mode:
                description:
                    - Dial mode (dial-on-demand or always-connect).
                choices:
                    - dial-on-demand
                    - always-connect
            dial-status:
                description:
                    - Dial status.
            ext-name:
                description:
                    - FortiExtender name.
            ha-shared-secret:
                description:
                    - HA shared secret.
            id:
                description:
                    - FortiExtender serial number.
                required: true
            ifname:
                description:
                    - FortiExtender interface name.
            initiated-update:
                description:
                    - Allow/disallow network initiated updates to the MODEM.
                choices:
                    - enable
                    - disable
            mode:
                description:
                    - FortiExtender mode.
                choices:
                    - standalone
                    - redundant
            modem-passwd:
                description:
                    - MODEM password.
            modem-type:
                description:
                    - MODEM type (CDMA, GSM/LTE or WIMAX).
                choices:
                    - cdma
                    - gsm/lte
                    - wimax
            multi-mode:
                description:
                    - MODEM mode of operation(3G,LTE,etc).
                choices:
                    - auto
                    - auto-3g
                    - force-lte
                    - force-3g
                    - force-2g
            ppp-auth-protocol:
                description:
                    - PPP authentication protocol (PAP,CHAP or auto).
                choices:
                    - auto
                    - pap
                    - chap
            ppp-echo-request:
                description:
                    - Enable/disable PPP echo request.
                choices:
                    - enable
                    - disable
            ppp-password:
                description:
                    - PPP password.
            ppp-username:
                description:
                    - PPP username.
            primary-ha:
                description:
                    - Primary HA.
            quota-limit-mb:
                description:
                    - Monthly quota limit (MB).
            redial:
                description:
                    - Number of redials allowed based on failed attempts.
                choices:
                    - none
                    - 1
                    - 2
                    - 3
                    - 4
                    - 5
                    - 6
                    - 7
                    - 8
                    - 9
                    - 10
            redundant-intf:
                description:
                    - Redundant interface.
            roaming:
                description:
                    - Enable/disable MODEM roaming.
                choices:
                    - enable
                    - disable
            role:
                description:
                    - FortiExtender work role(Primary, Secondary, None).
                choices:
                    - none
                    - primary
                    - secondary
            secondary-ha:
                description:
                    - Secondary HA.
            sim-pin:
                description:
                    - SIM PIN.
            vdom:
                description:
                    - VDOM
            wimax-auth-protocol:
                description:
                    - WiMax authentication protocol(TLS or TTLS).
                choices:
                    - tls
                    - ttls
            wimax-carrier:
                description:
                    - WiMax carrier.
            wimax-realm:
                description:
                    - WiMax realm.
'''

EXAMPLES = '''
- hosts: localhost
  vars:
   host: "192.168.122.40"
   username: "admin"
   password: ""
   vdom: "root"
  tasks:
  - name: Extender controller configuration.
    fortios_extender_controller_extender:
      host:  "{{ host }}"
      username: "{{ username }}"
      password: "{{ password }}"
      vdom:  "{{ vdom }}"
      extender_controller_extender:
        state: "present"
        aaa-shared-secret: "<your_own_value>"
        access-point-name: "<your_own_value>"
        admin: "disable"
        at-dial-script: "<your_own_value>"
        billing-start-day: "7"
        cdma-aaa-spi: "<your_own_value>"
        cdma-ha-spi: "<your_own_value>"
        cdma-nai: "<your_own_value>"
        conn-status: "11"
        description: "<your_own_value>"
        dial-mode: "dial-on-demand"
        dial-status: "14"
        ext-name: "<your_own_value>"
        ha-shared-secret: "<your_own_value>"
        id:  "17"
        ifname: "<your_own_value>"
        initiated-update: "enable"
        mode: "standalone"
        modem-passwd: "<your_own_value>"
        modem-type: "cdma"
        multi-mode: "auto"
        ppp-auth-protocol: "auto"
        ppp-echo-request: "enable"
        ppp-password: "<your_own_value>"
        ppp-username: "<your_own_value>"
        primary-ha: "<your_own_value>"
        quota-limit-mb: "29"
        redial: "none"
        redundant-intf: "<your_own_value>"
        roaming: "enable"
        role: "none"
        secondary-ha: "<your_own_value>"
        sim-pin: "<your_own_value>"
        vdom: "36"
        wimax-auth-protocol: "tls"
        wimax-carrier: "<your_own_value>"
        wimax-realm: "<your_own_value>"
'''

RETURN = '''
build:
  description: Build number of the fortigate image
  returned: always
  type: str
  sample: '1547'
http_method:
  description: Last method used to provision the content into FortiGate
  returned: always
  type: str
  sample: 'PUT'
http_status:
  description: Last result given by FortiGate on last operation applied
  returned: always
  type: str
  sample: "200"
mkey:
  description: Master key (id) used in the last call to FortiGate
  returned: success
  type: str
  sample: "id"
name:
  description: Name of the table used to fulfill the request
  returned: always
  type: str
  sample: "urlfilter"
path:
  description: Path of the table used to fulfill the request
  returned: always
  type: str
  sample: "webfilter"
revision:
  description: Internal revision number
  returned: always
  type: str
  sample: "17.0.2.10658"
serial:
  description: Serial number of the unit
  returned: always
  type: str
  sample: "FGVMEVYYQT3AB5352"
status:
  description: Indication of the operation's result
  returned: always
  type: str
  sample: "success"
vdom:
  description: Virtual domain used
  returned: always
  type: str
  sample: "root"
version:
  description: Version of the FortiGate
  returned: always
  type: str
  sample: "v5.6.3"

'''

from ansible.module_utils.basic import AnsibleModule

fos = None


def login(data):
    host = data['host']
    username = data['username']
    password = data['password']

    fos.debug('on')
    if 'https' in data and not data['https']:
        fos.https('off')
    else:
        fos.https('on')

    fos.login(host, username, password)


def filter_extender_controller_extender_data(json):
    option_list = ['aaa-shared-secret', 'access-point-name', 'admin',
                   'at-dial-script', 'billing-start-day', 'cdma-aaa-spi',
                   'cdma-ha-spi', 'cdma-nai', 'conn-status',
                   'description', 'dial-mode', 'dial-status',
                   'ext-name', 'ha-shared-secret', 'id',
                   'ifname', 'initiated-update', 'mode',
                   'modem-passwd', 'modem-type', 'multi-mode',
                   'ppp-auth-protocol', 'ppp-echo-request', 'ppp-password',
                   'ppp-username', 'primary-ha', 'quota-limit-mb',
                   'redial', 'redundant-intf', 'roaming',
                   'role', 'secondary-ha', 'sim-pin',
                   'vdom', 'wimax-auth-protocol', 'wimax-carrier',
                   'wimax-realm']
    dictionary = {}

    for attribute in option_list:
        if attribute in json and json[attribute] is not None:
            dictionary[attribute] = json[attribute]

    return dictionary


def extender_controller_extender(data, fos):
    vdom = data['vdom']
    extender_controller_extender_data = data['extender_controller_extender']
    filtered_data = filter_extender_controller_extender_data(extender_controller_extender_data)
    if extender_controller_extender_data['state'] == "present":
        return fos.set('extender-controller',
                       'extender',
                       data=filtered_data,
                       vdom=vdom)

    elif extender_controller_extender_data['state'] == "absent":
        return fos.delete('extender-controller',
                          'extender',
                          mkey=filtered_data['id'],
                          vdom=vdom)


def fortios_extender_controller(data, fos):
    login(data)

    methodlist = ['extender_controller_extender']
    for method in methodlist:
        if data[method]:
            resp = eval(method)(data, fos)
            break

    fos.logout()
    return not resp['status'] == "success", resp['status'] == "success", resp


def main():
    fields = {
        "host": {"required": True, "type": "str"},
        "username": {"required": True, "type": "str"},
        "password": {"required": False, "type": "str", "no_log": True},
        "vdom": {"required": False, "type": "str", "default": "root"},
        "https": {"required": False, "type": "bool", "default": "False"},
        "extender_controller_extender": {
            "required": False, "type": "dict",
            "options": {
                "state": {"required": True, "type": "str",
                          "choices": ["present", "absent"]},
                "aaa-shared-secret": {"required": False, "type": "str"},
                "access-point-name": {"required": False, "type": "str"},
                "admin": {"required": False, "type": "str",
                          "choices": ["disable", "discovered", "enable"]},
                "at-dial-script": {"required": False, "type": "str"},
                "billing-start-day": {"required": False, "type": "int"},
                "cdma-aaa-spi": {"required": False, "type": "str"},
                "cdma-ha-spi": {"required": False, "type": "str"},
                "cdma-nai": {"required": False, "type": "str"},
                "conn-status": {"required": False, "type": "int"},
                "description": {"required": False, "type": "str"},
                "dial-mode": {"required": False, "type": "str",
                              "choices": ["dial-on-demand", "always-connect"]},
                "dial-status": {"required": False, "type": "int"},
                "ext-name": {"required": False, "type": "str"},
                "ha-shared-secret": {"required": False, "type": "str"},
                "id": {"required": True, "type": "str"},
                "ifname": {"required": False, "type": "str"},
                "initiated-update": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "mode": {"required": False, "type": "str",
                         "choices": ["standalone", "redundant"]},
                "modem-passwd": {"required": False, "type": "str"},
                "modem-type": {"required": False, "type": "str",
                               "choices": ["cdma", "gsm/lte", "wimax"]},
                "multi-mode": {"required": False, "type": "str",
                               "choices": ["auto", "auto-3g", "force-lte",
                                           "force-3g", "force-2g"]},
                "ppp-auth-protocol": {"required": False, "type": "str",
                                      "choices": ["auto", "pap", "chap"]},
                "ppp-echo-request": {"required": False, "type": "str",
                                     "choices": ["enable", "disable"]},
                "ppp-password": {"required": False, "type": "str"},
                "ppp-username": {"required": False, "type": "str"},
                "primary-ha": {"required": False, "type": "str"},
                "quota-limit-mb": {"required": False, "type": "int"},
                "redial": {"required": False, "type": "str",
                           "choices": ["none", "1", "2",
                                       "3", "4", "5",
                                       "6", "7", "8",
                                       "9", "10"]},
                "redundant-intf": {"required": False, "type": "str"},
                "roaming": {"required": False, "type": "str",
                            "choices": ["enable", "disable"]},
                "role": {"required": False, "type": "str",
                         "choices": ["none", "primary", "secondary"]},
                "secondary-ha": {"required": False, "type": "str"},
                "sim-pin": {"required": False, "type": "str"},
                "vdom": {"required": False, "type": "int"},
                "wimax-auth-protocol": {"required": False, "type": "str",
                                        "choices": ["tls", "ttls"]},
                "wimax-carrier": {"required": False, "type": "str"},
                "wimax-realm": {"required": False, "type": "str"}

            }
        }
    }

    module = AnsibleModule(argument_spec=fields,
                           supports_check_mode=False)
    try:
        from fortiosapi import FortiOSAPI
    except ImportError:
        module.fail_json(msg="fortiosapi module is required")

    global fos
    fos = FortiOSAPI()

    is_error, has_changed, result = fortios_extender_controller(module.params, fos)

    if not is_error:
        module.exit_json(changed=has_changed, meta=result)
    else:
        module.fail_json(msg="Error in repo", meta=result)


if __name__ == '__main__':
    main()
